@@ -5,30 +5,32 @@ import re
 import typing as t
 
 from sqlglot import exp, generator, parser, tokens, transforms
-from sqlglot._typing import E
 from sqlglot.dialects.dialect import (
     Dialect,
+    NormalizationStrategy,
     arg_max_or_min_no_count,
     binary_from_function,
     date_add_interval_sql,
     datestrtodate_sql,
-    format_time_lambda,
+    build_formatted_time,
+    filter_array_using_unnest,
     if_sql,
     inline_array_sql,
-    json_keyvalue_comma_sql,
     max_or_greatest,
     min_or_least,
     no_ilike_sql,
-    parse_date_delta_with_interval,
+    build_date_delta_with_interval,
     regexp_replace_sql,
     rename_func,
     timestrtotime_sql,
     ts_or_ds_add_cast,
-    ts_or_ds_to_date_sql,
 )
 from sqlglot.helper import seq_get, split_num_words
 from sqlglot.tokens import TokenType
 
+if t.TYPE_CHECKING:
+    from sqlglot._typing import E, Lit
+
 logger = logging.getLogger("sqlglot")
 
 
@@ -36,47 +38,33 @@ def _derived_table_values_to_unnest(self: BigQuery.Generator, expression: exp.Va
     if not expression.find_ancestor(exp.From, exp.Join):
         return self.values_sql(expression)
 
+    structs = []
     alias = expression.args.get("alias")
+    for tup in expression.find_all(exp.Tuple):
+        field_aliases = alias.columns if alias else (f"_c{i}" for i in range(len(tup.expressions)))
+        expressions = [exp.alias_(fld, name) for fld, name in zip(tup.expressions, field_aliases)]
+        structs.append(exp.Struct(expressions=expressions))
 
-    structs = [
-        exp.Struct(
-            expressions=[
-                exp.alias_(value, column_name)
-                for value, column_name in zip(
-                    t.expressions,
-                    alias.columns
-                    if alias and alias.columns
-                    else (f"_c{i}" for i in range(len(t.expressions))),
-                )
-            ]
-        )
-        for t in expression.find_all(exp.Tuple)
-    ]
-
-    return self.unnest_sql(exp.Unnest(expressions=[exp.Array(expressions=structs)]))
+    return self.unnest_sql(exp.Unnest(expressions=[exp.array(*structs, copy=False)]))
 
 
 def _returnsproperty_sql(self: BigQuery.Generator, expression: exp.ReturnsProperty) -> str:
     this = expression.this
     if isinstance(this, exp.Schema):
-        this = f"{this.this} <{self.expressions(this)}>"
+        this = f"{self.sql(this, 'this')} <{self.expressions(this)}>"
     else:
         this = self.sql(this)
     return f"RETURNS {this}"
 
 
 def _create_sql(self: BigQuery.Generator, expression: exp.Create) -> str:
-    kind = expression.args["kind"]
     returns = expression.find(exp.ReturnsProperty)
-
-    if kind.upper() == "FUNCTION" and returns and returns.args.get("is_table"):
+    if expression.kind == "FUNCTION" and returns and returns.args.get("is_table"):
         expression.set("kind", "TABLE FUNCTION")
 
         if isinstance(expression.expression, (exp.Subquery, exp.Literal)):
             expression.set("expression", expression.expression.this)
 
-        return self.create_sql(expression)
-
     return self.create_sql(expression)
 
 
@@ -122,11 +110,10 @@ def _alias_ordered_group(expression: exp.Expression) -> exp.Expression:
                 if isinstance(select, exp.Alias)
             }
 
-            for e in group.expressions:
-                alias = aliases.get(e)
-
+            for grouped in group.expressions:
+                alias = aliases.get(grouped)
                 if alias:
-                    e.replace(exp.column(alias))
+                    grouped.replace(exp.column(alias))
 
     return expression
 
@@ -158,18 +145,24 @@ def _pushdown_cte_column_names(expression: exp.Expression) -> exp.Expression:
     return expression
 
 
-def _parse_timestamp(args: t.List) -> exp.StrToTime:
-    this = format_time_lambda(exp.StrToTime, "bigquery")([seq_get(args, 1), seq_get(args, 0)])
+def _build_parse_timestamp(args: t.List) -> exp.StrToTime:
+    this = build_formatted_time(exp.StrToTime, "bigquery")([seq_get(args, 1), seq_get(args, 0)])
     this.set("zone", seq_get(args, 2))
     return this
 
 
-def _parse_date(args: t.List) -> exp.Date | exp.DateFromParts:
+def _build_timestamp(args: t.List) -> exp.Timestamp:
+    timestamp = exp.Timestamp.from_arg_list(args)
+    timestamp.set("with_tz", True)
+    return timestamp
+
+
+def _build_date(args: t.List) -> exp.Date | exp.DateFromParts:
     expr_type = exp.DateFromParts if len(args) == 3 else exp.Date
     return expr_type.from_arg_list(args)
 
 
-def _parse_to_hex(args: t.List) -> exp.Hex | exp.MD5:
+def _build_to_hex(args: t.List) -> exp.Hex | exp.MD5:
     # TO_HEX(MD5(..)) is common in BigQuery, so it's parsed into MD5 to simplify its transpilation
     arg = seq_get(args, 0)
     return exp.MD5(this=arg.this) if isinstance(arg, exp.MD5Digest) else exp.Hex(this=arg)
@@ -198,29 +191,37 @@ def _ts_or_ds_diff_sql(self: BigQuery.Generator, expression: exp.TsOrDsDiff) ->
 
 def _unix_to_time_sql(self: BigQuery.Generator, expression: exp.UnixToTime) -> str:
     scale = expression.args.get("scale")
-    timestamp = self.sql(expression, "this")
-    if scale in (None, exp.UnixToTime.SECONDS):
-        return f"TIMESTAMP_SECONDS({timestamp})"
-    if scale == exp.UnixToTime.MILLIS:
-        return f"TIMESTAMP_MILLIS({timestamp})"
-    if scale == exp.UnixToTime.MICROS:
-        return f"TIMESTAMP_MICROS({timestamp})"
-    if scale == exp.UnixToTime.NANOS:
-        # We need to cast to INT64 because that's what BQ expects
-        return f"TIMESTAMP_MICROS(CAST({timestamp} / 1000 AS INT64))"
+    timestamp = expression.this
 
-    self.unsupported(f"Unsupported scale for timestamp: {scale}.")
-    return ""
+    if scale in (None, exp.UnixToTime.SECONDS):
+        return self.func("TIMESTAMP_SECONDS", timestamp)
+    if scale == exp.UnixToTime.MILLIS:
+        return self.func("TIMESTAMP_MILLIS", timestamp)
+    if scale == exp.UnixToTime.MICROS:
+        return self.func("TIMESTAMP_MICROS", timestamp)
+
+    unix_seconds = exp.cast(exp.Div(this=timestamp, expression=exp.func("POW", 10, scale)), "int64")
+    return self.func("TIMESTAMP_SECONDS", unix_seconds)
+
+
+def _build_time(args: t.List) -> exp.Func:
+    if len(args) == 1:
+        return exp.TsOrDsToTime(this=args[0])
+    if len(args) == 3:
+        return exp.TimeFromParts.from_arg_list(args)
+
+    return exp.Anonymous(this="TIME", expressions=args)
 
 
 class BigQuery(Dialect):
+    WEEK_OFFSET = -1
     UNNEST_COLUMN_ONLY = True
     SUPPORTS_USER_DEFINED_TYPES = False
     SUPPORTS_SEMI_ANTI_JOIN = False
     LOG_BASE_FIRST = False
 
     # https://cloud.google.com/bigquery/docs/reference/standard-sql/lexical#case_sensitivity
-    RESOLVES_IDENTIFIERS_AS_UPPERCASE = None
+    NORMALIZATION_STRATEGY = NormalizationStrategy.CASE_INSENSITIVE
 
     # bigquery udfs are case sensitive
     NORMALIZE_FUNCTIONS = False
@@ -259,8 +260,7 @@ class BigQuery(Dialect):
     # https://cloud.google.com/bigquery/docs/querying-partitioned-tables#query_an_ingestion-time_partitioned_table
     PSEUDOCOLUMNS = {"_PARTITIONTIME", "_PARTITIONDATE"}
 
-    @classmethod
-    def normalize_identifier(cls, expression: E) -> E:
+    def normalize_identifier(self, expression: E) -> E:
         if isinstance(expression, exp.Identifier):
             parent = expression.parent
             while isinstance(parent, exp.Dot):
@@ -302,6 +302,7 @@ class BigQuery(Dialect):
             "BYTES": TokenType.BINARY,
             "CURRENT_DATETIME": TokenType.CURRENT_DATETIME,
             "DECLARE": TokenType.COMMAND,
+            "EXCEPTION": TokenType.COMMAND,
             "FLOAT64": TokenType.DOUBLE,
             "FOR SYSTEM_TIME": TokenType.TIMESTAMP_SNAPSHOT,
             "MODEL": TokenType.MODEL,
@@ -310,6 +311,7 @@ class BigQuery(Dialect):
             "TIMESTAMP": TokenType.TIMESTAMPTZ,
         }
         KEYWORDS.pop("DIV")
+        KEYWORDS.pop("VALUES")
 
     class Parser(parser.Parser):
         PREFIXED_PIVOT_COLUMNS = True
@@ -318,23 +320,29 @@ class BigQuery(Dialect):
 
         FUNCTIONS = {
             **parser.Parser.FUNCTIONS,
-            "DATE": _parse_date,
-            "DATE_ADD": parse_date_delta_with_interval(exp.DateAdd),
-            "DATE_SUB": parse_date_delta_with_interval(exp.DateSub),
+            "DATE": _build_date,
+            "DATE_ADD": build_date_delta_with_interval(exp.DateAdd),
+            "DATE_SUB": build_date_delta_with_interval(exp.DateSub),
             "DATE_TRUNC": lambda args: exp.DateTrunc(
                 unit=exp.Literal.string(str(seq_get(args, 1))),
                 this=seq_get(args, 0),
             ),
-            "DATETIME_ADD": parse_date_delta_with_interval(exp.DatetimeAdd),
-            "DATETIME_SUB": parse_date_delta_with_interval(exp.DatetimeSub),
+            "DATETIME_ADD": build_date_delta_with_interval(exp.DatetimeAdd),
+            "DATETIME_SUB": build_date_delta_with_interval(exp.DatetimeSub),
             "DIV": binary_from_function(exp.IntDiv),
+            "FORMAT_DATE": lambda args: exp.TimeToStr(
+                this=exp.TsOrDsToDate(this=seq_get(args, 1)), format=seq_get(args, 0)
+            ),
             "GENERATE_ARRAY": exp.GenerateSeries.from_arg_list,
+            "JSON_EXTRACT_SCALAR": lambda args: exp.JSONExtractScalar(
+                this=seq_get(args, 0), expression=seq_get(args, 1) or exp.Literal.string("$")
+            ),
             "MD5": exp.MD5Digest.from_arg_list,
-            "TO_HEX": _parse_to_hex,
-            "PARSE_DATE": lambda args: format_time_lambda(exp.StrToDate, "bigquery")(
+            "TO_HEX": _build_to_hex,
+            "PARSE_DATE": lambda args: build_formatted_time(exp.StrToDate, "bigquery")(
                 [seq_get(args, 1), seq_get(args, 0)]
             ),
-            "PARSE_TIMESTAMP": _parse_timestamp,
+            "PARSE_TIMESTAMP": _build_parse_timestamp,
             "REGEXP_CONTAINS": exp.RegexpLike.from_arg_list,
             "REGEXP_EXTRACT": lambda args: exp.RegexpExtract(
                 this=seq_get(args, 0),
@@ -350,19 +358,19 @@ class BigQuery(Dialect):
                 this=seq_get(args, 0),
                 expression=seq_get(args, 1) or exp.Literal.string(","),
             ),
-            "TIME_ADD": parse_date_delta_with_interval(exp.TimeAdd),
-            "TIME_SUB": parse_date_delta_with_interval(exp.TimeSub),
-            "TIMESTAMP_ADD": parse_date_delta_with_interval(exp.TimestampAdd),
-            "TIMESTAMP_SUB": parse_date_delta_with_interval(exp.TimestampSub),
+            "TIME": _build_time,
+            "TIME_ADD": build_date_delta_with_interval(exp.TimeAdd),
+            "TIME_SUB": build_date_delta_with_interval(exp.TimeSub),
+            "TIMESTAMP": _build_timestamp,
+            "TIMESTAMP_ADD": build_date_delta_with_interval(exp.TimestampAdd),
+            "TIMESTAMP_SUB": build_date_delta_with_interval(exp.TimestampSub),
             "TIMESTAMP_MICROS": lambda args: exp.UnixToTime(
                 this=seq_get(args, 0), scale=exp.UnixToTime.MICROS
             ),
             "TIMESTAMP_MILLIS": lambda args: exp.UnixToTime(
                 this=seq_get(args, 0), scale=exp.UnixToTime.MILLIS
             ),
-            "TIMESTAMP_SECONDS": lambda args: exp.UnixToTime(
-                this=seq_get(args, 0), scale=exp.UnixToTime.SECONDS
-            ),
+            "TIMESTAMP_SECONDS": lambda args: exp.UnixToTime(this=seq_get(args, 0)),
             "TO_JSON_STRING": exp.JSONFormat.from_arg_list,
         }
 
@@ -382,11 +390,6 @@ class BigQuery(Dialect):
             TokenType.TABLE,
         }
 
-        ID_VAR_TOKENS = {
-            *parser.Parser.ID_VAR_TOKENS,
-            TokenType.VALUES,
-        }
-
         PROPERTY_PARSERS = {
             **parser.Parser.PROPERTY_PARSERS,
             "NOT DETERMINISTIC": lambda self: self.expression(
@@ -401,7 +404,7 @@ class BigQuery(Dialect):
         }
 
         RANGE_PARSERS = parser.Parser.RANGE_PARSERS.copy()
-        RANGE_PARSERS.pop(TokenType.OVERLAPS, None)
+        RANGE_PARSERS.pop(TokenType.OVERLAPS)
 
         NULL_TOKENS = {TokenType.NULL, TokenType.UNKNOWN}
 
@@ -411,6 +414,13 @@ class BigQuery(Dialect):
             TokenType.FOR: lambda self: self._parse_for_in(),
         }
 
+        BRACKET_OFFSETS = {
+            "OFFSET": (0, False),
+            "ORDINAL": (1, False),
+            "SAFE_OFFSET": (0, True),
+            "SAFE_ORDINAL": (1, True),
+        }
+
         def _parse_for_in(self) -> exp.ForIn:
             this = self._parse_range()
             self._match_text_seq("DO")
@@ -430,19 +440,20 @@ class BigQuery(Dialect):
             elif isinstance(this, exp.Literal):
                 table_name = this.name
 
-                if (
-                    self._curr
-                    and self._prev.end == self._curr.start - 1
-                    and self._parse_var(any_token=True)
-                ):
+                if self._is_connected() and self._parse_var(any_token=True):
                     table_name += self._prev.text
 
                 this = exp.Identifier(this=table_name, quoted=True)
 
             return this
 
-        def _parse_table_parts(self, schema: bool = False) -> exp.Table:
-            table = super()._parse_table_parts(schema=schema)
+        def _parse_table_parts(
+            self, schema: bool = False, is_db_reference: bool = False, wildcard: bool = False
+        ) -> exp.Table:
+            table = super()._parse_table_parts(
+                schema=schema, is_db_reference=is_db_reference, wildcard=True
+            )
+
             if isinstance(table.this, exp.Identifier) and "." in table.name:
                 catalog, db, this, *rest = (
                     t.cast(t.Optional[exp.Expression], exp.to_identifier(x))
@@ -456,7 +467,15 @@ class BigQuery(Dialect):
 
             return table
 
-        def _parse_json_object(self) -> exp.JSONObject:
+        @t.overload
+        def _parse_json_object(self, agg: Lit[False]) -> exp.JSONObject:
+            ...
+
+        @t.overload
+        def _parse_json_object(self, agg: Lit[True]) -> exp.JSONObjectAgg:
+            ...
+
+        def _parse_json_object(self, agg=False):
             json_object = super()._parse_json_object()
             array_kv_pair = seq_get(json_object.expressions, 0)
 
@@ -477,6 +496,26 @@ class BigQuery(Dialect):
 
             return json_object
 
+        def _parse_bracket(self, this: t.Optional[exp.Expression]) -> t.Optional[exp.Expression]:
+            bracket = super()._parse_bracket(this)
+
+            if this is bracket:
+                return bracket
+
+            if isinstance(bracket, exp.Bracket):
+                for expression in bracket.expressions:
+                    name = expression.name.upper()
+
+                    if name not in self.BRACKET_OFFSETS:
+                        break
+
+                    offset, safe = self.BRACKET_OFFSETS[name]
+                    bracket.set("offset", offset)
+                    bracket.set("safe", safe)
+                    expression.replace(expression.expressions[0])
+
+            return bracket
+
     class Generator(generator.Generator):
         EXPLICIT_UNION = True
         INTERVAL_ALLOWS_PLURAL_FORM = False
@@ -489,6 +528,14 @@ class BigQuery(Dialect):
         UNNEST_WITH_ORDINALITY = False
         COLLATE_IS_FUNC = True
         LIMIT_ONLY_LITERALS = True
+        SUPPORTS_TABLE_ALIAS_COLUMNS = False
+        UNPIVOT_ALIASES_ARE_IDENTIFIERS = False
+        JSON_KEY_VALUE_PAIR_SEP = ","
+        NULL_ORDERING_SUPPORTED = False
+        IGNORE_NULLS_IN_FUNC = True
+        JSON_PATH_SINGLE_QUOTE_ESCAPE = True
+        CAN_IMPLEMENT_ARRAY_ANY = True
+        NAMED_PLACEHOLDER_TOKEN = "@"
 
         TRANSFORMS = {
             **generator.Generator.TRANSFORMS,
@@ -496,21 +543,31 @@ class BigQuery(Dialect):
             exp.ArgMax: arg_max_or_min_no_count("MAX_BY"),
             exp.ArgMin: arg_max_or_min_no_count("MIN_BY"),
             exp.ArrayContains: _array_contains_sql,
+            exp.ArrayFilter: filter_array_using_unnest,
             exp.ArraySize: rename_func("ARRAY_LENGTH"),
             exp.Cast: transforms.preprocess([transforms.remove_precision_parameterized_types]),
-            exp.CollateProperty: lambda self, e: f"DEFAULT COLLATE {self.sql(e, 'this')}"
-            if e.args.get("default")
-            else f"COLLATE {self.sql(e, 'this')}",
+            exp.CollateProperty: lambda self, e: (
+                f"DEFAULT COLLATE {self.sql(e, 'this')}"
+                if e.args.get("default")
+                else f"COLLATE {self.sql(e, 'this')}"
+            ),
+            exp.Commit: lambda *_: "COMMIT TRANSACTION",
+            exp.CountIf: rename_func("COUNTIF"),
             exp.Create: _create_sql,
             exp.CTE: transforms.preprocess([_pushdown_cte_column_names]),
             exp.DateAdd: date_add_interval_sql("DATE", "ADD"),
-            exp.DateDiff: lambda self, e: f"DATE_DIFF({self.sql(e, 'this')}, {self.sql(e, 'expression')}, {self.sql(e.args.get('unit', 'DAY'))})",
+            exp.DateDiff: lambda self, e: self.func(
+                "DATE_DIFF", e.this, e.expression, e.unit or "DAY"
+            ),
             exp.DateFromParts: rename_func("DATE"),
             exp.DateStrToDate: datestrtodate_sql,
             exp.DateSub: date_add_interval_sql("DATE", "SUB"),
             exp.DatetimeAdd: date_add_interval_sql("DATETIME", "ADD"),
             exp.DatetimeSub: date_add_interval_sql("DATETIME", "SUB"),
             exp.DateTrunc: lambda self, e: self.func("DATE_TRUNC", e.this, e.text("unit")),
+            exp.FromTimeZone: lambda self, e: self.func(
+                "DATETIME", self.func("TIMESTAMP", e.this, e.args.get("zone")), "'UTC'"
+            ),
             exp.GenerateSeries: rename_func("GENERATE_ARRAY"),
             exp.GroupConcat: rename_func("STRING_AGG"),
             exp.Hex: rename_func("TO_HEX"),
@@ -518,7 +575,6 @@ class BigQuery(Dialect):
             exp.ILike: no_ilike_sql,
             exp.IntDiv: rename_func("DIV"),
             exp.JSONFormat: rename_func("TO_JSON_STRING"),
-            exp.JSONKeyValue: json_keyvalue_comma_sql,
             exp.Max: max_or_greatest,
             exp.MD5: lambda self, e: self.func("TO_HEX", self.func("MD5", e.this)),
             exp.MD5Digest: rename_func("MD5"),
@@ -534,6 +590,7 @@ class BigQuery(Dialect):
             exp.RegexpReplace: regexp_replace_sql,
             exp.RegexpLike: rename_func("REGEXP_CONTAINS"),
             exp.ReturnsProperty: _returnsproperty_sql,
+            exp.Rollback: lambda *_: "ROLLBACK TRANSACTION",
             exp.Select: transforms.preprocess(
                 [
                     transforms.explode_to_unnest(),
@@ -544,31 +601,40 @@ class BigQuery(Dialect):
                 ]
             ),
             exp.SHA2: lambda self, e: self.func(
-                f"SHA256" if e.text("length") == "256" else "SHA512", e.this
+                "SHA256" if e.text("length") == "256" else "SHA512", e.this
             ),
-            exp.StabilityProperty: lambda self, e: f"DETERMINISTIC"
-            if e.name == "IMMUTABLE"
-            else "NOT DETERMINISTIC",
-            exp.StrToDate: lambda self, e: f"PARSE_DATE({self.format_time(e)}, {self.sql(e, 'this')})",
+            exp.StabilityProperty: lambda self, e: (
+                "DETERMINISTIC" if e.name == "IMMUTABLE" else "NOT DETERMINISTIC"
+            ),
+            exp.StrToDate: lambda self, e: self.func("PARSE_DATE", self.format_time(e), e.this),
             exp.StrToTime: lambda self, e: self.func(
                 "PARSE_TIMESTAMP", self.format_time(e), e.this, e.args.get("zone")
             ),
             exp.TimeAdd: date_add_interval_sql("TIME", "ADD"),
+            exp.TimeFromParts: rename_func("TIME"),
             exp.TimeSub: date_add_interval_sql("TIME", "SUB"),
             exp.TimestampAdd: date_add_interval_sql("TIMESTAMP", "ADD"),
+            exp.TimestampDiff: rename_func("TIMESTAMP_DIFF"),
             exp.TimestampSub: date_add_interval_sql("TIMESTAMP", "SUB"),
             exp.TimeStrToTime: timestrtotime_sql,
-            exp.TimeToStr: lambda self, e: f"FORMAT_DATE({self.format_time(e)}, {self.sql(e, 'this')})",
-            exp.Trim: lambda self, e: self.func(f"TRIM", e.this, e.expression),
+            exp.Transaction: lambda *_: "BEGIN TRANSACTION",
+            exp.Trim: lambda self, e: self.func("TRIM", e.this, e.expression),
             exp.TsOrDsAdd: _ts_or_ds_add_sql,
             exp.TsOrDsDiff: _ts_or_ds_diff_sql,
-            exp.TsOrDsToDate: ts_or_ds_to_date_sql("bigquery"),
+            exp.TsOrDsToTime: rename_func("TIME"),
             exp.Unhex: rename_func("FROM_HEX"),
+            exp.UnixDate: rename_func("UNIX_DATE"),
             exp.UnixToTime: _unix_to_time_sql,
             exp.Values: _derived_table_values_to_unnest,
             exp.VariancePop: rename_func("VAR_POP"),
         }
 
+        SUPPORTED_JSON_PATH_PARTS = {
+            exp.JSONPathKey,
+            exp.JSONPathRoot,
+            exp.JSONPathSubscript,
+        }
+
         TYPE_MAPPING = {
             **generator.Generator.TYPE_MAPPING,
             exp.DataType.Type.BIGDECIMAL: "BIGNUMERIC",
@@ -700,10 +766,27 @@ class BigQuery(Dialect):
             "within",
         }
 
+        def timetostr_sql(self, expression: exp.TimeToStr) -> str:
+            this = expression.this if isinstance(expression.this, exp.TsOrDsToDate) else expression
+            return self.func("FORMAT_DATE", self.format_time(expression), this.this)
+
+        def struct_sql(self, expression: exp.Struct) -> str:
+            args = []
+            for expr in expression.expressions:
+                if isinstance(expr, self.KEY_VALUE_DEFINITIONS):
+                    arg = f"{self.sql(expr, 'expression')} AS {expr.this.name}"
+                else:
+                    arg = self.sql(expr)
+
+                args.append(arg)
+
+            return self.func("STRUCT", *args)
+
         def eq_sql(self, expression: exp.EQ) -> str:
             # Operands of = cannot be NULL in BigQuery
             if isinstance(expression.left, exp.Null) or isinstance(expression.right, exp.Null):
-                return "NULL"
+                if not isinstance(expression.parent, exp.Update):
+                    return "NULL"
 
             return self.binary(expression, "=")
 
@@ -722,11 +805,6 @@ class BigQuery(Dialect):
         def trycast_sql(self, expression: exp.TryCast) -> str:
             return self.cast_sql(expression, safe_prefix="SAFE_")
 
-        def cte_sql(self, expression: exp.CTE) -> str:
-            if expression.alias_column_names:
-                self.unsupported("Column names in CTE definition are not supported.")
-            return super().cte_sql(expression)
-
         def array_sql(self, expression: exp.Array) -> str:
             first_arg = seq_get(expression.expressions, 0)
             if isinstance(first_arg, exp.Subqueryable):
@@ -734,25 +812,46 @@ class BigQuery(Dialect):
 
             return inline_array_sql(self, expression)
 
-        def transaction_sql(self, *_) -> str:
-            return "BEGIN TRANSACTION"
+        def bracket_sql(self, expression: exp.Bracket) -> str:
+            this = self.sql(expression, "this")
+            expressions = expression.expressions
 
-        def commit_sql(self, *_) -> str:
-            return "COMMIT TRANSACTION"
+            if len(expressions) == 1:
+                arg = expressions[0]
+                if arg.type is None:
+                    from sqlglot.optimizer.annotate_types import annotate_types
 
-        def rollback_sql(self, *_) -> str:
-            return "ROLLBACK TRANSACTION"
+                    arg = annotate_types(arg)
+
+                if arg.type and arg.type.this in exp.DataType.TEXT_TYPES:
+                    # BQ doesn't support bracket syntax with string values
+                    return f"{this}.{arg.name}"
+
+            expressions_sql = ", ".join(self.sql(e) for e in expressions)
+            offset = expression.args.get("offset")
+
+            if offset == 0:
+                expressions_sql = f"OFFSET({expressions_sql})"
+            elif offset == 1:
+                expressions_sql = f"ORDINAL({expressions_sql})"
+            elif offset is not None:
+                self.unsupported(f"Unsupported array offset: {offset}")
+
+            if expression.args.get("safe"):
+                expressions_sql = f"SAFE_{expressions_sql}"
+
+            return f"{this}[{expressions_sql}]"
 
         def in_unnest_op(self, expression: exp.Unnest) -> str:
             return self.sql(expression)
 
         def except_op(self, expression: exp.Except) -> str:
-            if not expression.args.get("distinct", False):
+            if not expression.args.get("distinct"):
                 self.unsupported("EXCEPT without DISTINCT is not supported in BigQuery")
             return f"EXCEPT{' DISTINCT' if expression.args.get('distinct') else ' ALL'}"
 
         def intersect_op(self, expression: exp.Intersect) -> str:
-            if not expression.args.get("distinct", False):
+            if not expression.args.get("distinct"):
                 self.unsupported("INTERSECT without DISTINCT is not supported in BigQuery")
             return f"INTERSECT{' DISTINCT' if expression.args.get('distinct') else ' ALL'}"
 
