@@ -3,13 +3,19 @@ from __future__ import annotations
 from sqlglot import exp, transforms
 from sqlglot.dialects.dialect import (
     date_delta_sql,
-    parse_date_delta,
+    build_date_delta,
     timestamptrunc_sql,
 )
 from sqlglot.dialects.spark import Spark
 from sqlglot.tokens import TokenType
 
 
+def _timestamp_diff(
+    self: Databricks.Generator, expression: exp.DatetimeDiff | exp.TimestampDiff
+) -> str:
+    return self.func("TIMESTAMPDIFF", expression.unit, expression.expression, expression.this)
+
+
 class Databricks(Spark):
     SAFE_DIVISION = False
 
@@ -19,9 +25,10 @@ class Databricks(Spark):
 
         FUNCTIONS = {
             **Spark.Parser.FUNCTIONS,
-            "DATEADD": parse_date_delta(exp.DateAdd),
-            "DATE_ADD": parse_date_delta(exp.DateAdd),
-            "DATEDIFF": parse_date_delta(exp.DateDiff),
+            "DATEADD": build_date_delta(exp.DateAdd),
+            "DATE_ADD": build_date_delta(exp.DateAdd),
+            "DATEDIFF": build_date_delta(exp.DateDiff),
+            "TIMESTAMPDIFF": build_date_delta(exp.TimestampDiff),
         }
 
         FACTOR = {
@@ -30,22 +37,23 @@ class Databricks(Spark):
         }
 
     class Generator(Spark.Generator):
+        TABLESAMPLE_SEED_KEYWORD = "REPEATABLE"
+
         TRANSFORMS = {
             **Spark.Generator.TRANSFORMS,
             exp.DateAdd: date_delta_sql("DATEADD"),
             exp.DateDiff: date_delta_sql("DATEDIFF"),
             exp.DatetimeAdd: lambda self, e: self.func(
-                "TIMESTAMPADD", e.text("unit"), e.expression, e.this
+                "TIMESTAMPADD", e.unit, e.expression, e.this
             ),
             exp.DatetimeSub: lambda self, e: self.func(
                 "TIMESTAMPADD",
-                e.text("unit"),
+                e.unit,
                 exp.Mul(this=e.expression, expression=exp.Literal.number(-1)),
                 e.this,
             ),
-            exp.DatetimeDiff: lambda self, e: self.func(
-                "TIMESTAMPDIFF", e.text("unit"), e.expression, e.this
-            ),
+            exp.DatetimeDiff: _timestamp_diff,
+            exp.TimestampDiff: _timestamp_diff,
             exp.DatetimeTrunc: timestamptrunc_sql,
             exp.JSONExtract: lambda self, e: self.binary(e, ":"),
             exp.Select: transforms.preprocess(
@@ -69,6 +77,7 @@ class Databricks(Spark):
             ):
                 # only BIGINT generated identity constraints are supported
                 expression.set("kind", exp.DataType.build("bigint"))
+
             return super().columndef_sql(expression, sep)
 
         def generatedasidentitycolumnconstraint_sql(
