@@ -5,7 +5,8 @@ import typing as t
 from sqlglot import exp, generator, parser, tokens, transforms
 from sqlglot.dialects.dialect import (
     Dialect,
-    format_time_lambda,
+    NormalizationStrategy,
+    build_formatted_time,
     no_ilike_sql,
     rename_func,
     trim_sql,
@@ -17,26 +18,7 @@ if t.TYPE_CHECKING:
     from sqlglot._typing import E
 
 
-def _parse_xml_table(self: Oracle.Parser) -> exp.XMLTable:
-    this = self._parse_string()
-
-    passing = None
-    columns = None
-
-    if self._match_text_seq("PASSING"):
-        # The BY VALUE keywords are optional and are provided for semantic clarity
-        self._match_text_seq("BY", "VALUE")
-        passing = self._parse_csv(self._parse_column)
-
-    by_ref = self._match_text_seq("RETURNING", "SEQUENCE", "BY", "REF")
-
-    if self._match_text_seq("COLUMNS"):
-        columns = self._parse_csv(self._parse_field_def)
-
-    return self.expression(exp.XMLTable, this=this, passing=passing, columns=columns, by_ref=by_ref)
-
-
-def to_char(args: t.List) -> exp.TimeToStr | exp.ToChar:
+def _build_timetostr_or_tochar(args: t.List) -> exp.TimeToStr | exp.ToChar:
     this = seq_get(args, 0)
 
     if this and not this.type:
@@ -44,7 +26,7 @@ def to_char(args: t.List) -> exp.TimeToStr | exp.ToChar:
 
         annotate_types(this)
         if this.is_type(*exp.DataType.TEMPORAL_TYPES):
-            return format_time_lambda(exp.TimeToStr, "oracle", default=True)(args)
+            return build_formatted_time(exp.TimeToStr, "oracle", default=True)(args)
 
     return exp.ToChar.from_arg_list(args)
 
@@ -52,10 +34,10 @@ def to_char(args: t.List) -> exp.TimeToStr | exp.ToChar:
 class Oracle(Dialect):
     ALIAS_POST_TABLESAMPLE = True
     LOCKING_READS_SUPPORTED = True
+    TABLESAMPLE_SIZE_IS_PERCENT = True
 
     # See section 8: https://docs.oracle.com/cd/A97630_01/server.920/a96540/sql_elements9a.htm
-    RESOLVES_IDENTIFIERS_AS_UPPERCASE = True
-    ALTER_TABLE_ADD_COLUMN_KEYWORD = False
+    NORMALIZATION_STRATEGY = NormalizationStrategy.UPPERCASE
 
     # https://docs.oracle.com/database/121/SQLRF/sql_elements004.htm#SQLRF00212
     # https://docs.python.org/3/library/datetime.html#strftime-and-strptime-format-codes
@@ -81,15 +63,20 @@ class Oracle(Dialect):
         "WW": "%W",  # Week of year (1-53)
         "YY": "%y",  # 15
         "YYYY": "%Y",  # 2015
+        "FF6": "%f",  # only 6 digits are supported in python formats
     }
 
     class Parser(parser.Parser):
+        ALTER_TABLE_ADD_REQUIRED_FOR_EACH_COLUMN = False
         WINDOW_BEFORE_PAREN_TOKENS = {TokenType.OVER, TokenType.KEEP}
+        VALUES_FOLLOWED_BY_PAREN = False
 
         FUNCTIONS = {
             **parser.Parser.FUNCTIONS,
             "SQUARE": lambda args: exp.Pow(this=seq_get(args, 0), expression=exp.Literal.number(2)),
-            "TO_CHAR": to_char,
+            "TO_CHAR": _build_timetostr_or_tochar,
+            "TO_TIMESTAMP": build_formatted_time(exp.StrToTime, "oracle"),
+            "TO_DATE": build_formatted_time(exp.StrToDate, "oracle"),
         }
 
         FUNCTION_PARSERS: t.Dict[str, t.Callable] = {
@@ -103,7 +90,12 @@ class Oracle(Dialect):
                 this=self._parse_format_json(self._parse_bitwise()),
                 order=self._parse_order(),
             ),
-            "XMLTABLE": _parse_xml_table,
+            "XMLTABLE": lambda self: self._parse_xml_table(),
+        }
+
+        QUERY_MODIFIER_PARSERS = {
+            **parser.Parser.QUERY_MODIFIER_PARSERS,
+            TokenType.ORDER_SIBLINGS_BY: lambda self: ("order", self._parse_order()),
         }
 
         TYPE_LITERAL_PARSERS = {
@@ -116,6 +108,26 @@ class Oracle(Dialect):
         # Reference: https://stackoverflow.com/a/336455
         DISTINCT_TOKENS = {TokenType.DISTINCT, TokenType.UNIQUE}
 
+        def _parse_xml_table(self) -> exp.XMLTable:
+            this = self._parse_string()
+
+            passing = None
+            columns = None
+
+            if self._match_text_seq("PASSING"):
+                # The BY VALUE keywords are optional and are provided for semantic clarity
+                self._match_text_seq("BY", "VALUE")
+                passing = self._parse_csv(self._parse_column)
+
+            by_ref = self._match_text_seq("RETURNING", "SEQUENCE", "BY", "REF")
+
+            if self._match_text_seq("COLUMNS"):
+                columns = self._parse_csv(self._parse_field_def)
+
+            return self.expression(
+                exp.XMLTable, this=this, passing=passing, columns=columns, by_ref=by_ref
+            )
+
         def _parse_json_array(self, expr_type: t.Type[E], **kwargs) -> E:
             return self.expression(
                 expr_type,
@@ -151,9 +163,12 @@ class Oracle(Dialect):
         TABLE_HINTS = False
         COLUMN_JOIN_MARKS_SUPPORTED = True
         DATA_TYPE_SPECIFIERS_ALLOWED = True
-        ALTER_TABLE_ADD_COLUMN_KEYWORD = False
-
+        ALTER_TABLE_INCLUDE_COLUMN_KEYWORD = False
         LIMIT_FETCH = "FETCH"
+        TABLESAMPLE_KEYWORDS = "SAMPLE"
+        LAST_DAY_SUPPORTS_DATE_PART = False
+        SUPPORTS_SELECT_INTO = True
+        TZ_TO_WITH_TIME_ZONE = True
 
         TYPE_MAPPING = {
             **generator.Generator.TYPE_MAPPING,
@@ -167,6 +182,8 @@ class Oracle(Dialect):
             exp.DataType.Type.NVARCHAR: "NVARCHAR2",
             exp.DataType.Type.NCHAR: "NCHAR",
             exp.DataType.Type.TEXT: "CLOB",
+            exp.DataType.Type.TIMETZ: "TIME",
+            exp.DataType.Type.TIMESTAMPTZ: "TIMESTAMP",
             exp.DataType.Type.BINARY: "BLOB",
             exp.DataType.Type.VARBINARY: "BLOB",
         }
@@ -184,15 +201,17 @@ class Oracle(Dialect):
                     transforms.eliminate_qualify,
                 ]
             ),
-            exp.StrToTime: lambda self, e: f"TO_TIMESTAMP({self.sql(e, 'this')}, {self.format_time(e)})",
+            exp.StrToTime: lambda self, e: self.func("TO_TIMESTAMP", e.this, self.format_time(e)),
+            exp.StrToDate: lambda self, e: self.func("TO_DATE", e.this, self.format_time(e)),
             exp.Subquery: lambda self, e: self.subquery_sql(e, sep=" "),
             exp.Substring: rename_func("SUBSTR"),
             exp.Table: lambda self, e: self.table_sql(e, sep=" "),
             exp.TableSample: lambda self, e: self.tablesample_sql(e, sep=" "),
-            exp.TimeToStr: lambda self, e: f"TO_CHAR({self.sql(e, 'this')}, {self.format_time(e)})",
+            exp.TimeToStr: lambda self, e: self.func("TO_CHAR", e.this, self.format_time(e)),
             exp.ToChar: lambda self, e: self.function_fallback_sql(e),
             exp.Trim: trim_sql,
-            exp.UnixToTime: lambda self, e: f"TO_DATE('1970-01-01','YYYY-MM-DD') + ({self.sql(e, 'this')} / 86400)",
+            exp.UnixToTime: lambda self,
+            e: f"TO_DATE('1970-01-01', 'YYYY-MM-DD') + ({self.sql(e, 'this')} / 86400)",
         }
 
         PROPERTIES_LOCATION = {
@@ -200,6 +219,10 @@ class Oracle(Dialect):
             exp.VolatileProperty: exp.Properties.Location.UNSUPPORTED,
         }
 
+        def currenttimestamp_sql(self, expression: exp.CurrentTimestamp) -> str:
+            this = expression.this
+            return self.func("CURRENT_TIMESTAMP", this) if this else "CURRENT_TIMESTAMP"
+
         def offset_sql(self, expression: exp.Offset) -> str:
             return f"{super().offset_sql(expression)} ROWS"
 
@@ -232,8 +255,10 @@ class Oracle(Dialect):
             "MATCH_RECOGNIZE": TokenType.MATCH_RECOGNIZE,
             "MINUS": TokenType.EXCEPT,
             "NVARCHAR2": TokenType.NVARCHAR,
+            "ORDER SIBLINGS BY": TokenType.ORDER_SIBLINGS_BY,
             "SAMPLE": TokenType.TABLE_SAMPLE,
             "START": TokenType.BEGIN,
+            "SYSDATE": TokenType.CURRENT_TIMESTAMP,
             "TOP": TokenType.TOP,
             "VARCHAR2": TokenType.VARCHAR,
         }
