@@ -13,10 +13,10 @@ from sqlglot.dialects.dialect import (
     generatedasidentitycolumnconstraint_sql,
     max_or_greatest,
     min_or_least,
-    parse_date_delta,
+    build_date_delta,
     rename_func,
     timestrtotime_sql,
-    ts_or_ds_to_date_sql,
+    trim_sql,
 )
 from sqlglot.expressions import DataType
 from sqlglot.helper import seq_get
@@ -64,10 +64,10 @@ DEFAULT_START_DATE = datetime.date(1900, 1, 1)
 BIT_TYPES = {exp.EQ, exp.NEQ, exp.Is, exp.In, exp.Select, exp.Alias}
 
 
-def _format_time_lambda(
+def _build_formatted_time(
     exp_class: t.Type[E], full_format_mapping: t.Optional[bool] = None
 ) -> t.Callable[[t.List], E]:
-    def _format_time(args: t.List) -> E:
+    def _builder(args: t.List) -> E:
         assert len(args) == 2
 
         return exp_class(
@@ -75,17 +75,19 @@ def _format_time_lambda(
             format=exp.Literal.string(
                 format_time(
                     args[0].name.lower(),
-                    {**TSQL.TIME_MAPPING, **FULL_FORMAT_TIME_MAPPING}
-                    if full_format_mapping
-                    else TSQL.TIME_MAPPING,
+                    (
+                        {**TSQL.TIME_MAPPING, **FULL_FORMAT_TIME_MAPPING}
+                        if full_format_mapping
+                        else TSQL.TIME_MAPPING
+                    ),
                 )
             ),
         )
 
-    return _format_time
+    return _builder
 
 
-def _parse_format(args: t.List) -> exp.Expression:
+def _build_format(args: t.List) -> exp.NumberToStr | exp.TimeToStr:
     this = seq_get(args, 0)
     fmt = seq_get(args, 1)
     culture = seq_get(args, 2)
@@ -105,21 +107,20 @@ def _parse_format(args: t.List) -> exp.Expression:
     return exp.TimeToStr(this=this, format=fmt, culture=culture)
 
 
-def _parse_eomonth(args: t.List) -> exp.Expression:
-    date = seq_get(args, 0)
+def _build_eomonth(args: t.List) -> exp.LastDay:
+    date = exp.TsOrDsToDate(this=seq_get(args, 0))
     month_lag = seq_get(args, 1)
-    unit = DATE_DELTA_INTERVAL.get("month")
 
     if month_lag is None:
-        return exp.LastDateOfMonth(this=date)
+        this: exp.Expression = date
+    else:
+        unit = DATE_DELTA_INTERVAL.get("month")
+        this = exp.DateAdd(this=date, expression=month_lag, unit=unit and exp.var(unit))
 
-    # Remove month lag argument in parser as its compared with the number of arguments of the resulting class
-    args.remove(month_lag)
-
-    return exp.LastDateOfMonth(this=exp.DateAdd(this=date, expression=month_lag, unit=unit))
+    return exp.LastDay(this=this)
 
 
-def _parse_hashbytes(args: t.List) -> exp.Expression:
+def _build_hashbytes(args: t.List) -> exp.Expression:
     kind, data = args
     kind = kind.name.upper() if kind.is_string else ""
 
@@ -137,26 +138,27 @@ def _parse_hashbytes(args: t.List) -> exp.Expression:
     return exp.func("HASHBYTES", *args)
 
 
-DATEPART_ONLY_FORMATS = {"dw", "hour", "quarter"}
+DATEPART_ONLY_FORMATS = {"DW", "HOUR", "QUARTER"}
 
 
 def _format_sql(self: TSQL.Generator, expression: exp.NumberToStr | exp.TimeToStr) -> str:
-    fmt = (
-        expression.args["format"]
-        if isinstance(expression, exp.NumberToStr)
-        else exp.Literal.string(
-            format_time(
-                expression.text("format"),
-                t.cast(t.Dict[str, str], TSQL.INVERSE_TIME_MAPPING),
-            )
-        )
-    )
+    fmt = expression.args["format"]
 
-    # There is no format for "quarter"
-    if fmt.name.lower() in DATEPART_ONLY_FORMATS:
-        return self.func("DATEPART", fmt.name, expression.this)
+    if not isinstance(expression, exp.NumberToStr):
+        if fmt.is_string:
+            mapped_fmt = format_time(fmt.name, TSQL.INVERSE_TIME_MAPPING)
 
-    return self.func("FORMAT", expression.this, fmt, expression.args.get("culture"))
+            name = (mapped_fmt or "").upper()
+            if name in DATEPART_ONLY_FORMATS:
+                return self.func("DATEPART", name, expression.this)
+
+            fmt_sql = self.sql(exp.Literal.string(mapped_fmt))
+        else:
+            fmt_sql = self.format_time(expression) or self.sql(fmt)
+    else:
+        fmt_sql = self.sql(fmt)
+
+    return self.func("FORMAT", expression.this, fmt_sql, expression.args.get("culture"))
 
 
 def _string_agg_sql(self: TSQL.Generator, expression: exp.GroupConcat) -> str:
@@ -177,10 +179,10 @@ def _string_agg_sql(self: TSQL.Generator, expression: exp.GroupConcat) -> str:
     return f"STRING_AGG({self.format_args(this, separator)}){order}"
 
 
-def _parse_date_delta(
+def _build_date_delta(
     exp_class: t.Type[E], unit_mapping: t.Optional[t.Dict[str, str]] = None
 ) -> t.Callable[[t.List], E]:
-    def inner_func(args: t.List) -> E:
+    def _builder(args: t.List) -> E:
         unit = seq_get(args, 0)
         if unit and unit_mapping:
             unit = exp.var(unit_mapping.get(unit.name.lower(), unit.name))
@@ -202,7 +204,7 @@ def _parse_date_delta(
             unit=unit,
         )
 
-    return inner_func
+    return _builder
 
 
 def qualify_derived_table_outputs(expression: exp.Expression) -> exp.Expression:
@@ -239,6 +241,58 @@ def qualify_derived_table_outputs(expression: exp.Expression) -> exp.Expression:
     return expression
 
 
+# https://learn.microsoft.com/en-us/sql/t-sql/functions/datetimefromparts-transact-sql?view=sql-server-ver16#syntax
+def _build_datetimefromparts(args: t.List) -> exp.TimestampFromParts:
+    return exp.TimestampFromParts(
+        year=seq_get(args, 0),
+        month=seq_get(args, 1),
+        day=seq_get(args, 2),
+        hour=seq_get(args, 3),
+        min=seq_get(args, 4),
+        sec=seq_get(args, 5),
+        milli=seq_get(args, 6),
+    )
+
+
+# https://learn.microsoft.com/en-us/sql/t-sql/functions/timefromparts-transact-sql?view=sql-server-ver16#syntax
+def _build_timefromparts(args: t.List) -> exp.TimeFromParts:
+    return exp.TimeFromParts(
+        hour=seq_get(args, 0),
+        min=seq_get(args, 1),
+        sec=seq_get(args, 2),
+        fractions=seq_get(args, 3),
+        precision=seq_get(args, 4),
+    )
+
+
+def _build_with_arg_as_text(
+    klass: t.Type[exp.Expression],
+) -> t.Callable[[t.List[exp.Expression]], exp.Expression]:
+    def _parse(args: t.List[exp.Expression]) -> exp.Expression:
+        this = seq_get(args, 0)
+
+        if this and not this.is_string:
+            this = exp.cast(this, exp.DataType.Type.TEXT)
+
+        expression = seq_get(args, 1)
+        kwargs = {"this": this}
+
+        if expression:
+            kwargs["expression"] = expression
+
+        return klass(**kwargs)
+
+    return _parse
+
+
+def _json_extract_sql(
+    self: TSQL.Generator, expression: exp.JSONExtract | exp.JSONExtractScalar
+) -> str:
+    json_query = self.func("JSON_QUERY", expression.this, expression.expression)
+    json_value = self.func("JSON_VALUE", expression.this, expression.expression)
+    return self.func("ISNULL", json_query, json_value)
+
+
 class TSQL(Dialect):
     NORMALIZATION_STRATEGY = NormalizationStrategy.CASE_INSENSITIVE
     TIME_FORMAT = "'yyyy-mm-dd hh:mm:ss'"
@@ -352,7 +406,7 @@ class TSQL(Dialect):
     }
 
     class Tokenizer(tokens.Tokenizer):
-        IDENTIFIERS = ['"', ("[", "]")]
+        IDENTIFIERS = [("[", "]"), '"']
         QUOTES = ["'", '"']
         HEX_STRINGS = [("0x", ""), ("0X", "")]
         VAR_SINGLE_TOKENS = {"@", "$", "#"}
@@ -362,6 +416,7 @@ class TSQL(Dialect):
             "DATETIME2": TokenType.DATETIME,
             "DATETIMEOFFSET": TokenType.TIMESTAMPTZ,
             "DECLARE": TokenType.COMMAND,
+            "EXEC": TokenType.COMMAND,
             "IMAGE": TokenType.IMAGE,
             "MONEY": TokenType.MONEY,
             "NTEXT": TokenType.TEXT,
@@ -393,24 +448,28 @@ class TSQL(Dialect):
                 substr=seq_get(args, 0),
                 position=seq_get(args, 2),
             ),
-            "DATEADD": parse_date_delta(exp.DateAdd, unit_mapping=DATE_DELTA_INTERVAL),
-            "DATEDIFF": _parse_date_delta(exp.DateDiff, unit_mapping=DATE_DELTA_INTERVAL),
-            "DATENAME": _format_time_lambda(exp.TimeToStr, full_format_mapping=True),
-            "DATEPART": _format_time_lambda(exp.TimeToStr),
-            "EOMONTH": _parse_eomonth,
-            "FORMAT": _parse_format,
+            "DATEADD": build_date_delta(exp.DateAdd, unit_mapping=DATE_DELTA_INTERVAL),
+            "DATEDIFF": _build_date_delta(exp.DateDiff, unit_mapping=DATE_DELTA_INTERVAL),
+            "DATENAME": _build_formatted_time(exp.TimeToStr, full_format_mapping=True),
+            "DATEPART": _build_formatted_time(exp.TimeToStr),
+            "DATETIMEFROMPARTS": _build_datetimefromparts,
+            "EOMONTH": _build_eomonth,
+            "FORMAT": _build_format,
             "GETDATE": exp.CurrentTimestamp.from_arg_list,
-            "HASHBYTES": _parse_hashbytes,
-            "IIF": exp.If.from_arg_list,
+            "HASHBYTES": _build_hashbytes,
             "ISNULL": exp.Coalesce.from_arg_list,
-            "JSON_VALUE": exp.JSONExtractScalar.from_arg_list,
-            "LEN": exp.Length.from_arg_list,
+            "JSON_QUERY": parser.build_extract_json_with_path(exp.JSONExtract),
+            "JSON_VALUE": parser.build_extract_json_with_path(exp.JSONExtractScalar),
+            "LEN": _build_with_arg_as_text(exp.Length),
+            "LEFT": _build_with_arg_as_text(exp.Left),
+            "RIGHT": _build_with_arg_as_text(exp.Right),
             "REPLICATE": exp.Repeat.from_arg_list,
             "SQUARE": lambda args: exp.Pow(this=seq_get(args, 0), expression=exp.Literal.number(2)),
             "SYSDATETIME": exp.CurrentTimestamp.from_arg_list,
             "SUSER_NAME": exp.CurrentUser.from_arg_list,
             "SUSER_SNAME": exp.CurrentUser.from_arg_list,
             "SYSTEM_USER": exp.CurrentUser.from_arg_list,
+            "TIMEFROMPARTS": _build_timefromparts,
         }
 
         JOIN_HINTS = {
@@ -440,6 +499,8 @@ class TSQL(Dialect):
         LOG_DEFAULTS_TO_LN = True
 
         ALTER_TABLE_ADD_REQUIRED_FOR_EACH_COLUMN = False
+        STRING_ALIASES = True
+        NO_PAREN_IF_COMMANDS = False
 
         def _parse_projections(self) -> t.List[exp.Expression]:
             """
@@ -449,9 +510,11 @@ class TSQL(Dialect):
             See: https://learn.microsoft.com/en-us/sql/t-sql/queries/select-clause-transact-sql?view=sql-server-ver16#syntax
             """
             return [
-                exp.alias_(projection.expression, projection.this.this, copy=False)
-                if isinstance(projection, exp.EQ) and isinstance(projection.this, exp.Column)
-                else projection
+                (
+                    exp.alias_(projection.expression, projection.this.this, copy=False)
+                    if isinstance(projection, exp.EQ) and isinstance(projection.this, exp.Column)
+                    else projection
+                )
                 for projection in super()._parse_projections()
             ]
 
@@ -630,8 +693,11 @@ class TSQL(Dialect):
         COMPUTED_COLUMN_WITH_TYPE = False
         CTE_RECURSIVE_KEYWORD_REQUIRED = False
         ENSURE_BOOLS = True
-        NULL_ORDERING_SUPPORTED = False
+        NULL_ORDERING_SUPPORTED = None
         SUPPORTS_SINGLE_ARG_CONCAT = False
+        TABLESAMPLE_SEED_KEYWORD = "REPEATABLE"
+        SUPPORTS_SELECT_INTO = True
+        JSON_PATH_BRACKETED_KEY_SUPPORTED = False
 
         EXPRESSIONS_WITHOUT_NESTED_CTES = {
             exp.Delete,
@@ -643,6 +709,12 @@ class TSQL(Dialect):
             exp.Update,
         }
 
+        SUPPORTED_JSON_PATH_PARTS = {
+            exp.JSONPathKey,
+            exp.JSONPathRoot,
+            exp.JSONPathSubscript,
+        }
+
         TYPE_MAPPING = {
             **generator.Generator.TYPE_MAPPING,
             exp.DataType.Type.BOOLEAN: "BIT",
@@ -669,11 +741,14 @@ class TSQL(Dialect):
             exp.GeneratedAsIdentityColumnConstraint: generatedasidentitycolumnconstraint_sql,
             exp.GroupConcat: _string_agg_sql,
             exp.If: rename_func("IIF"),
-            exp.Length: rename_func("LEN"),
+            exp.JSONExtract: _json_extract_sql,
+            exp.JSONExtractScalar: _json_extract_sql,
+            exp.LastDay: lambda self, e: self.func("EOMONTH", e.this),
             exp.Max: max_or_greatest,
             exp.MD5: lambda self, e: self.func("HASHBYTES", exp.Literal.string("MD5"), e.this),
             exp.Min: min_or_least,
             exp.NumberToStr: _format_sql,
+            exp.ParseJSON: lambda self, e: self.sql(e, "this"),
             exp.Select: transforms.preprocess(
                 [
                     transforms.eliminate_distinct_on,
@@ -681,6 +756,9 @@ class TSQL(Dialect):
                     transforms.eliminate_qualify,
                 ]
             ),
+            exp.StrPosition: lambda self, e: self.func(
+                "CHARINDEX", e.args.get("substr"), e.this, e.args.get("position")
+            ),
             exp.Subquery: transforms.preprocess([qualify_derived_table_outputs]),
             exp.SHA: lambda self, e: self.func("HASHBYTES", exp.Literal.string("SHA1"), e.this),
             exp.SHA2: lambda self, e: self.func(
@@ -689,9 +767,9 @@ class TSQL(Dialect):
             exp.TemporaryProperty: lambda self, e: "",
             exp.TimeStrToTime: timestrtotime_sql,
             exp.TimeToStr: _format_sql,
+            exp.Trim: trim_sql,
             exp.TsOrDsAdd: date_delta_sql("DATEADD", cast=True),
             exp.TsOrDsDiff: date_delta_sql("DATEDIFF"),
-            exp.TsOrDsToDate: ts_or_ds_to_date_sql("tsql"),
         }
 
         TRANSFORMS.pop(exp.ReturnsProperty)
@@ -701,6 +779,46 @@ class TSQL(Dialect):
             exp.VolatileProperty: exp.Properties.Location.UNSUPPORTED,
         }
 
+        def lateral_op(self, expression: exp.Lateral) -> str:
+            cross_apply = expression.args.get("cross_apply")
+            if cross_apply is True:
+                return "CROSS APPLY"
+            if cross_apply is False:
+                return "OUTER APPLY"
+
+            # TODO: perhaps we can check if the parent is a Join and transpile it appropriately
+            self.unsupported("LATERAL clause is not supported.")
+            return "LATERAL"
+
+        def timefromparts_sql(self, expression: exp.TimeFromParts) -> str:
+            nano = expression.args.get("nano")
+            if nano is not None:
+                nano.pop()
+                self.unsupported("Specifying nanoseconds is not supported in TIMEFROMPARTS.")
+
+            if expression.args.get("fractions") is None:
+                expression.set("fractions", exp.Literal.number(0))
+            if expression.args.get("precision") is None:
+                expression.set("precision", exp.Literal.number(0))
+
+            return rename_func("TIMEFROMPARTS")(self, expression)
+
+        def timestampfromparts_sql(self, expression: exp.TimestampFromParts) -> str:
+            zone = expression.args.get("zone")
+            if zone is not None:
+                zone.pop()
+                self.unsupported("Time zone is not supported in DATETIMEFROMPARTS.")
+
+            nano = expression.args.get("nano")
+            if nano is not None:
+                nano.pop()
+                self.unsupported("Specifying nanoseconds is not supported in DATETIMEFROMPARTS.")
+
+            if expression.args.get("milli") is None:
+                expression.set("milli", exp.Literal.number(0))
+
+            return rename_func("DATETIMEFROMPARTS")(self, expression)
+
         def set_operation(self, expression: exp.Union, op: str) -> str:
             limit = expression.args.get("limit")
             if limit:
@@ -740,19 +858,25 @@ class TSQL(Dialect):
             return sql
 
         def create_sql(self, expression: exp.Create) -> str:
-            kind = self.sql(expression, "kind").upper()
+            kind = expression.kind
             exists = expression.args.pop("exists", None)
             sql = super().create_sql(expression)
 
+            like_property = expression.find(exp.LikeProperty)
+            if like_property:
+                ctas_expression = like_property.this
+            else:
+                ctas_expression = expression.expression
+
             table = expression.find(exp.Table)
 
             # Convert CTAS statement to SELECT .. INTO ..
-            if kind == "TABLE" and expression.expression:
-                ctas_with = expression.expression.args.get("with")
+            if kind == "TABLE" and ctas_expression:
+                ctas_with = ctas_expression.args.get("with")
                 if ctas_with:
                     ctas_with = ctas_with.pop()
 
-                subquery = expression.expression
+                subquery = ctas_expression
                 if isinstance(subquery, exp.Subqueryable):
                     subquery = subquery.subquery()
 
@@ -760,6 +884,9 @@ class TSQL(Dialect):
                 select_into.set("into", exp.Into(this=table))
                 select_into.set("with", ctas_with)
 
+                if like_property:
+                    select_into.limit(0, copy=False)
+
                 sql = self.sql(select_into)
 
             if exists:
@@ -848,3 +975,21 @@ class TSQL(Dialect):
             this = self.sql(expression, "this")
             expressions = self.expressions(expression, flat=True, sep=" ")
             return f"CONSTRAINT {this} {expressions}"
+
+        def length_sql(self, expression: exp.Length) -> str:
+            return self._uncast_text(expression, "LEN")
+
+        def right_sql(self, expression: exp.Right) -> str:
+            return self._uncast_text(expression, "RIGHT")
+
+        def left_sql(self, expression: exp.Left) -> str:
+            return self._uncast_text(expression, "LEFT")
+
+        def _uncast_text(self, expression: exp.Expression, name: str) -> str:
+            this = expression.this
+            if isinstance(this, exp.Cast) and this.is_type(exp.DataType.Type.TEXT):
+                this_sql = self.sql(this, "this")
+            else:
+                this_sql = self.sql(this)
+            expression_sql = self.sql(expression, "expression")
+            return self.func(name, this_sql, expression_sql if expression_sql else None)
