@@ -6,7 +6,7 @@ import logging
 import re
 import sys
 import typing as t
-from collections.abc import Collection
+from collections.abc import Collection, Set
 from contextlib import contextmanager
 from copy import copy
 from enum import Enum
@@ -236,7 +236,7 @@ def tsort(dag: t.Dict[T, t.Set[T]]) -> t.List[T]:
 
     for node, deps in tuple(dag.items()):
         for dep in deps:
-            if not dep in dag:
+            if dep not in dag:
                 dag[dep] = set()
 
     while dag:
@@ -320,6 +320,14 @@ def find_new_name(taken: t.Collection[str], base: str) -> str:
     return new
 
 
+def is_int(text: str) -> bool:
+    try:
+        int(text)
+        return True
+    except ValueError:
+        return False
+
+
 def name_sequence(prefix: str) -> t.Callable[[], str]:
     """Returns a name generator given a prefix (e.g. a0, a1, a2, ... if the prefix is "a")."""
     sequence = count()
@@ -488,3 +496,31 @@ DATE_UNITS = {"day", "week", "month", "quarter", "year", "year_month"}
 
 def is_date_unit(expression: t.Optional[exp.Expression]) -> bool:
     return expression is not None and expression.name.lower() in DATE_UNITS
+
+
+K = t.TypeVar("K")
+V = t.TypeVar("V")
+
+
+class SingleValuedMapping(t.Mapping[K, V]):
+    """
+    Mapping where all keys return the same value.
+
+    This rigamarole is meant to avoid copying keys, which was originally intended
+    as an optimization while qualifying columns for tables with lots of columns.
+    """
+
+    def __init__(self, keys: t.Collection[K], value: V):
+        self._keys = keys if isinstance(keys, Set) else set(keys)
+        self._value = value
+
+    def __getitem__(self, key: K) -> V:
+        if key in self._keys:
+            return self._value
+        raise KeyError(key)
+
+    def __len__(self) -> int:
+        return len(self._keys)
+
+    def __iter__(self) -> t.Iterator[K]:
+        return iter(self._keys)
