@@ -41,9 +41,9 @@ class Node:
             else:
                 label = node.expression.sql(pretty=True, dialect=dialect)
                 source = node.source.transform(
-                    lambda n: exp.Tag(this=n, prefix="<b>", postfix="</b>")
-                    if n is node.expression
-                    else n,
+                    lambda n: (
+                        exp.Tag(this=n, prefix="<b>", postfix="</b>") if n is node.expression else n
+                    ),
                     copy=False,
                 ).sql(pretty=True, dialect=dialect)
                 title = f"<pre>{source}</pre>"
@@ -129,13 +129,10 @@ def lineage(
             if isinstance(column, int)
             else next(
                 (select for select in scope.expression.selects if select.alias_or_name == column),
-                exp.Star() if scope.expression.is_star else None,
+                exp.Star() if scope.expression.is_star else scope.expression,
             )
         )
 
-        if not select:
-            raise ValueError(f"Could not find {column} in {scope.expression}")
-
         if isinstance(scope.expression, exp.Union):
             upstream = upstream or Node(name="UNION", source=scope.expression, expression=select)
 
@@ -156,7 +153,7 @@ def lineage(
                 raise ValueError(f"Could not find {column} in {scope.expression}")
 
             for s in scope.union_scopes:
-                to_node(index, scope=s, upstream=upstream)
+                to_node(index, scope=s, upstream=upstream, alias=alias)
 
             return upstream
 
@@ -194,6 +191,8 @@ def lineage(
         # if the select is a star add all scope sources as downstreams
         if select.is_star:
             for source in scope.sources.values():
+                if isinstance(source, Scope):
+                    source = source.expression
                 node.downstream.append(Node(name=select.sql(), source=source, expression=source))
 
         # Find all columns that went into creating this one to list their lineage nodes.
@@ -210,7 +209,11 @@ def lineage(
             if isinstance(source, Scope):
                 # The table itself came from a more specific scope. Recurse into that one using the unaliased column name.
                 to_node(
-                    c.name, scope=source, scope_name=table, upstream=node, alias=aliases.get(table)
+                    c.name,
+                    scope=source,
+                    scope_name=table,
+                    upstream=node,
+                    alias=aliases.get(table) or alias,
                 )
             else:
                 # The source is not a scope - we've reached the end of the line. At this point, if a source is not found
