@@ -4,7 +4,6 @@ import functools
 import typing as t
 
 from sqlglot import exp
-from sqlglot._typing import E
 from sqlglot.helper import (
     ensure_list,
     is_date_unit,
@@ -17,7 +16,7 @@ from sqlglot.optimizer.scope import Scope, traverse_scope
 from sqlglot.schema import Schema, ensure_schema
 
 if t.TYPE_CHECKING:
-    B = t.TypeVar("B", bound=exp.Binary)
+    from sqlglot._typing import B, E
 
     BinaryCoercionFunc = t.Callable[[exp.Expression, exp.Expression], exp.DataType.Type]
     BinaryCoercions = t.Dict[
@@ -195,6 +194,9 @@ class TypeAnnotator(metaclass=_TypeAnnotator):
             exp.StrPosition,
             exp.TsOrDiToDi,
         },
+        exp.DataType.Type.JSON: {
+            exp.ParseJSON,
+        },
         exp.DataType.Type.TIMESTAMP: {
             exp.CurrentTime,
             exp.CurrentTimestamp,
@@ -202,7 +204,6 @@ class TypeAnnotator(metaclass=_TypeAnnotator):
             exp.TimeAdd,
             exp.TimeStrToTime,
             exp.TimeSub,
-            exp.Timestamp,
             exp.TimestampAdd,
             exp.TimestampSub,
             exp.UnixToTime,
@@ -261,6 +262,7 @@ class TypeAnnotator(metaclass=_TypeAnnotator):
         exp.DateTrunc: lambda self, e: self._annotate_timeunit(e),
         exp.Distinct: lambda self, e: self._annotate_by_args(e, "expressions"),
         exp.Div: lambda self, e: self._annotate_div(e),
+        exp.Explode: lambda self, e: self._annotate_explode(e),
         exp.Filter: lambda self, e: self._annotate_by_args(e, "this"),
         exp.If: lambda self, e: self._annotate_by_args(e, "true", "false"),
         exp.Interval: lambda self, e: self._annotate_with_type(e, exp.DataType.Type.INTERVAL),
@@ -273,8 +275,13 @@ class TypeAnnotator(metaclass=_TypeAnnotator):
         exp.Nullif: lambda self, e: self._annotate_by_args(e, "this", "expression"),
         exp.Slice: lambda self, e: self._annotate_with_type(e, exp.DataType.Type.UNKNOWN),
         exp.Sum: lambda self, e: self._annotate_by_args(e, "this", "expressions", promote=True),
+        exp.Timestamp: lambda self, e: self._annotate_with_type(
+            e,
+            exp.DataType.Type.TIMESTAMPTZ if e.args.get("with_tz") else exp.DataType.Type.TIMESTAMP,
+        ),
         exp.TryCast: lambda self, e: self._annotate_with_type(e, e.args["to"]),
         exp.VarMap: lambda self, e: self._annotate_with_type(e, exp.DataType.Type.MAP),
+        exp.Struct: lambda self, e: self._annotate_by_args(e, "expressions", struct=True),
     }
 
     NESTED_TYPES = {
@@ -330,9 +337,9 @@ class TypeAnnotator(metaclass=_TypeAnnotator):
         self._visited: t.Set[int] = set()
 
     def _set_type(
-        self, expression: exp.Expression, target_type: exp.DataType | exp.DataType.Type
+        self, expression: exp.Expression, target_type: t.Optional[exp.DataType | exp.DataType.Type]
     ) -> None:
-        expression.type = target_type  # type: ignore
+        expression.type = target_type or exp.DataType.Type.UNKNOWN  # type: ignore
         self._visited.add(id(expression))
 
     def annotate(self, expression: E) -> E:
@@ -475,9 +482,28 @@ class TypeAnnotator(metaclass=_TypeAnnotator):
         self._set_type(expression, target_type)
         return self._annotate_args(expression)
 
+    @t.no_type_check
+    def _annotate_struct_value(
+        self, expression: exp.Expression
+    ) -> t.Optional[exp.DataType] | exp.ColumnDef:
+        alias = expression.args.get("alias")
+        if alias:
+            return exp.ColumnDef(this=alias.copy(), kind=expression.type)
+
+        # Case: key = value or key := value
+        if expression.expression:
+            return exp.ColumnDef(this=expression.this.copy(), kind=expression.expression.type)
+
+        return expression.type
+
     @t.no_type_check
     def _annotate_by_args(
-        self, expression: E, *args: str, promote: bool = False, array: bool = False
+        self,
+        expression: E,
+        *args: str,
+        promote: bool = False,
+        array: bool = False,
+        struct: bool = False,
     ) -> E:
         self._annotate_args(expression)
 
@@ -488,7 +514,14 @@ class TypeAnnotator(metaclass=_TypeAnnotator):
 
         last_datatype = None
         for expr in expressions:
-            last_datatype = self._maybe_coerce(last_datatype or expr.type, expr.type)
+            expr_type = expr.type
+
+            # Stop at the first nested data type found - we don't want to _maybe_coerce nested types
+            if expr_type.args.get("nested"):
+                last_datatype = expr_type
+                break
+
+            last_datatype = self._maybe_coerce(last_datatype or expr_type, expr_type)
 
         self._set_type(expression, last_datatype or exp.DataType.Type.UNKNOWN)
 
@@ -506,6 +539,16 @@ class TypeAnnotator(metaclass=_TypeAnnotator):
                 ),
             )
 
+        if struct:
+            self._set_type(
+                expression,
+                exp.DataType(
+                    this=exp.DataType.Type.STRUCT,
+                    expressions=[self._annotate_struct_value(expr) for expr in expressions],
+                    nested=True,
+                ),
+            )
+
         return expression
 
     def _annotate_timeunit(
@@ -532,13 +575,11 @@ class TypeAnnotator(metaclass=_TypeAnnotator):
         if isinstance(bracket_arg, exp.Slice):
             self._set_type(expression, this.type)
         elif this.type.is_type(exp.DataType.Type.ARRAY):
-            contained_type = seq_get(this.type.expressions, 0) or exp.DataType.Type.UNKNOWN
-            self._set_type(expression, contained_type)
+            self._set_type(expression, seq_get(this.type.expressions, 0))
         elif isinstance(this, (exp.Map, exp.VarMap)) and bracket_arg in this.keys:
             index = this.keys.index(bracket_arg)
             value = seq_get(this.values, index)
-            value_type = value.type if value else exp.DataType.Type.UNKNOWN
-            self._set_type(expression, value_type or exp.DataType.Type.UNKNOWN)
+            self._set_type(expression, value.type if value else None)
         else:
             self._set_type(expression, exp.DataType.Type.UNKNOWN)
 
@@ -559,3 +600,8 @@ class TypeAnnotator(metaclass=_TypeAnnotator):
             self._set_type(expression, self._maybe_coerce(left_type, right_type))
 
         return expression
+
+    def _annotate_explode(self, expression: exp.Explode) -> exp.Explode:
+        self._annotate_args(expression)
+        self._set_type(expression, seq_get(expression.this.type.expressions, 0))
+        return expression
