@@ -4,19 +4,22 @@ import itertools
 import typing as t
 
 from sqlglot import alias, exp
-from sqlglot._typing import E
 from sqlglot.dialects.dialect import Dialect, DialectType
 from sqlglot.errors import OptimizeError
-from sqlglot.helper import seq_get
+from sqlglot.helper import seq_get, SingleValuedMapping
 from sqlglot.optimizer.scope import Scope, build_scope, traverse_scope, walk_in_scope
 from sqlglot.optimizer.simplify import simplify_parens
 from sqlglot.schema import Schema, ensure_schema
 
+if t.TYPE_CHECKING:
+    from sqlglot._typing import E
+
 
 def qualify_columns(
     expression: exp.Expression,
     schema: t.Dict | Schema,
     expand_alias_refs: bool = True,
+    expand_stars: bool = True,
     infer_schema: t.Optional[bool] = None,
 ) -> exp.Expression:
     """
@@ -32,11 +35,17 @@ def qualify_columns(
     Args:
         expression: Expression to qualify.
         schema: Database schema.
-        expand_alias_refs: Whether or not to expand references to aliases.
-        infer_schema: Whether or not to infer the schema if missing.
+        expand_alias_refs: Whether to expand references to aliases.
+        expand_stars: Whether to expand star queries. This is a necessary step
+            for most of the optimizer's rules to work; do not set to False unless you
+            know what you're doing!
+        infer_schema: Whether to infer the schema if missing.
 
     Returns:
         The qualified expression.
+
+    Notes:
+        - Currently only handles a single PIVOT or UNPIVOT operator
     """
     schema = ensure_schema(schema)
     infer_schema = schema.empty if infer_schema is None else infer_schema
@@ -57,7 +66,8 @@ def qualify_columns(
             _expand_alias_refs(scope, resolver)
 
         if not isinstance(scope.expression, exp.UDTF):
-            _expand_stars(scope, resolver, using_column_tables, pseudocolumns)
+            if expand_stars:
+                _expand_stars(scope, resolver, using_column_tables, pseudocolumns)
             qualify_outputs(scope)
 
         _expand_group_by(scope)
@@ -68,21 +78,41 @@ def qualify_columns(
 
 def validate_qualify_columns(expression: E) -> E:
     """Raise an `OptimizeError` if any columns aren't qualified"""
-    unqualified_columns = []
+    all_unqualified_columns = []
     for scope in traverse_scope(expression):
         if isinstance(scope.expression, exp.Select):
-            unqualified_columns.extend(scope.unqualified_columns)
+            unqualified_columns = scope.unqualified_columns
+
             if scope.external_columns and not scope.is_correlated_subquery and not scope.pivots:
                 column = scope.external_columns[0]
-                raise OptimizeError(
-                    f"""Column '{column}' could not be resolved{f" for table: '{column.table}'" if column.table else ''}"""
-                )
+                for_table = f" for table: '{column.table}'" if column.table else ""
+                raise OptimizeError(f"Column '{column}' could not be resolved{for_table}")
+
+            if unqualified_columns and scope.pivots and scope.pivots[0].unpivot:
+                # New columns produced by the UNPIVOT can't be qualified, but there may be columns
+                # under the UNPIVOT's IN clause that can and should be qualified. We recompute
+                # this list here to ensure those in the former category will be excluded.
+                unpivot_columns = set(_unpivot_columns(scope.pivots[0]))
+                unqualified_columns = [c for c in unqualified_columns if c not in unpivot_columns]
+
+            all_unqualified_columns.extend(unqualified_columns)
+
+    if all_unqualified_columns:
+        raise OptimizeError(f"Ambiguous columns: {all_unqualified_columns}")
 
-    if unqualified_columns:
-        raise OptimizeError(f"Ambiguous columns: {unqualified_columns}")
     return expression
 
 
+def _unpivot_columns(unpivot: exp.Pivot) -> t.Iterator[exp.Column]:
+    name_column = []
+    field = unpivot.args.get("field")
+    if isinstance(field, exp.In) and isinstance(field.this, exp.Column):
+        name_column.append(field.this)
+
+    value_columns = (c for e in unpivot.expressions for c in e.find_all(exp.Column))
+    return itertools.chain(name_column, value_columns)
+
+
 def _pop_table_column_aliases(derived_tables: t.List[exp.CTE | exp.Subquery]) -> None:
     """
     Remove table column aliases.
@@ -134,12 +164,7 @@ def _expand_using(scope: Scope, resolver: Resolver) -> t.Dict[str, t.Any]:
 
             table = table or source_table
             conditions.append(
-                exp.condition(
-                    exp.EQ(
-                        this=exp.column(identifier, table=table),
-                        expression=exp.column(identifier, table=join_table),
-                    )
-                )
+                exp.column(identifier, table=table).eq(exp.column(identifier, table=join_table))
             )
 
             # Set all values in the dict to None, because we only care about the key ordering
@@ -182,7 +207,7 @@ def _expand_alias_refs(scope: Scope, resolver: Resolver) -> None:
         if not node:
             return
 
-        for column, *_ in walk_in_scope(node):
+        for column, *_ in walk_in_scope(node, prune=lambda node, *_: node.is_star):
             if not isinstance(column, exp.Column):
                 continue
 
@@ -216,6 +241,7 @@ def _expand_alias_refs(scope: Scope, resolver: Resolver) -> None:
     replace_columns(expression.args.get("group"), literal_index=True)
     replace_columns(expression.args.get("having"), resolve_table=True)
     replace_columns(expression.args.get("qualify"), resolve_table=True)
+
     scope.clear_cache()
 
 
@@ -353,18 +379,25 @@ def _expand_stars(
     replace_columns: t.Dict[int, t.Dict[str, str]] = {}
     coalesced_columns = set()
 
-    # TODO: handle optimization of multiple PIVOTs (and possibly UNPIVOTs) in the future
-    pivot_columns = None
     pivot_output_columns = None
+    pivot_exclude_columns = None
+
     pivot = t.cast(t.Optional[exp.Pivot], seq_get(scope.pivots, 0))
+    if isinstance(pivot, exp.Pivot) and not pivot.alias_column_names:
+        if pivot.unpivot:
+            pivot_output_columns = [c.output_name for c in _unpivot_columns(pivot)]
 
-    has_pivoted_source = pivot and not pivot.args.get("unpivot")
-    if pivot and has_pivoted_source:
-        pivot_columns = set(col.output_name for col in pivot.find_all(exp.Column))
+            field = pivot.args.get("field")
+            if isinstance(field, exp.In):
+                pivot_exclude_columns = {
+                    c.output_name for e in field.expressions for c in e.find_all(exp.Column)
+                }
+        else:
+            pivot_exclude_columns = set(c.output_name for c in pivot.find_all(exp.Column))
 
-        pivot_output_columns = [col.output_name for col in pivot.args.get("columns", [])]
-        if not pivot_output_columns:
-            pivot_output_columns = [col.alias_or_name for col in pivot.expressions]
+            pivot_output_columns = [c.output_name for c in pivot.args.get("columns", [])]
+            if not pivot_output_columns:
+                pivot_output_columns = [c.alias_or_name for c in pivot.expressions]
 
     for expression in scope.expression.selects:
         if isinstance(expression, exp.Star):
@@ -384,47 +417,53 @@ def _expand_stars(
                 raise OptimizeError(f"Unknown table: {table}")
 
             columns = resolver.get_source_columns(table, only_visible=True)
+            columns = columns or scope.outer_column_list
 
             if pseudocolumns:
                 columns = [name for name in columns if name.upper() not in pseudocolumns]
 
-            if columns and "*" not in columns:
-                table_id = id(table)
-                columns_to_exclude = except_columns.get(table_id) or set()
+            if not columns or "*" in columns:
+                return
 
-                if pivot and has_pivoted_source and pivot_columns and pivot_output_columns:
-                    implicit_columns = [col for col in columns if col not in pivot_columns]
+            table_id = id(table)
+            columns_to_exclude = except_columns.get(table_id) or set()
+
+            if pivot:
+                if pivot_output_columns and pivot_exclude_columns:
+                    pivot_columns = [c for c in columns if c not in pivot_exclude_columns]
+                    pivot_columns.extend(pivot_output_columns)
+                else:
+                    pivot_columns = pivot.alias_column_names
+
+                if pivot_columns:
                     new_selections.extend(
                         exp.alias_(exp.column(name, table=pivot.alias), name, copy=False)
-                        for name in implicit_columns + pivot_output_columns
+                        for name in pivot_columns
                         if name not in columns_to_exclude
                     )
                     continue
 
-                for name in columns:
-                    if name in using_column_tables and table in using_column_tables[name]:
-                        if name in coalesced_columns:
-                            continue
+            for name in columns:
+                if name in columns_to_exclude or name in coalesced_columns:
+                    continue
+                if name in using_column_tables and table in using_column_tables[name]:
+                    coalesced_columns.add(name)
+                    tables = using_column_tables[name]
+                    coalesce = [exp.column(name, table=table) for table in tables]
 
-                        coalesced_columns.add(name)
-                        tables = using_column_tables[name]
-                        coalesce = [exp.column(name, table=table) for table in tables]
-
-                        new_selections.append(
-                            alias(
-                                exp.Coalesce(this=coalesce[0], expressions=coalesce[1:]),
-                                alias=name,
-                                copy=False,
-                            )
+                    new_selections.append(
+                        alias(
+                            exp.Coalesce(this=coalesce[0], expressions=coalesce[1:]),
+                            alias=name,
+                            copy=False,
                         )
-                    elif name not in columns_to_exclude:
-                        alias_ = replace_columns.get(table_id, {}).get(name, name)
-                        column = exp.column(name, table=table)
-                        new_selections.append(
-                            alias(column, alias_, copy=False) if alias_ != name else column
-                        )
-            else:
-                return
+                    )
+                else:
+                    alias_ = replace_columns.get(table_id, {}).get(name, name)
+                    column = exp.column(name, table=table)
+                    new_selections.append(
+                        alias(column, alias_, copy=False) if alias_ != name else column
+                    )
 
     # Ensures we don't overwrite the initial selections with an empty list
     if new_selections:
@@ -472,6 +511,9 @@ def qualify_outputs(scope_or_expression: Scope | exp.Expression) -> None:
     for i, (selection, aliased_column) in enumerate(
         itertools.zip_longest(scope.expression.selects, scope.outer_column_list)
     ):
+        if selection is None:
+            break
+
         if isinstance(selection, exp.Subquery):
             if not selection.output_name:
                 selection.set("alias", exp.TableAlias(this=exp.to_identifier(f"_col_{i}")))
@@ -479,6 +521,7 @@ def qualify_outputs(scope_or_expression: Scope | exp.Expression) -> None:
             selection = alias(
                 selection,
                 alias=selection.output_name or f"_col_{i}",
+                copy=False,
             )
         if aliased_column:
             selection.set("alias", exp.to_identifier(aliased_column))
@@ -495,6 +538,38 @@ def quote_identifiers(expression: E, dialect: DialectType = None, identify: bool
     )
 
 
+def pushdown_cte_alias_columns(expression: exp.Expression) -> exp.Expression:
+    """
+    Pushes down the CTE alias columns into the projection,
+
+    This step is useful in Snowflake where the CTE alias columns can be referenced in the HAVING.
+
+    Example:
+        >>> import sqlglot
+        >>> expression = sqlglot.parse_one("WITH y (c) AS (SELECT SUM(a) FROM ( SELECT 1 a ) AS x HAVING c > 0) SELECT c FROM y")
+        >>> pushdown_cte_alias_columns(expression).sql()
+        'WITH y(c) AS (SELECT SUM(a) AS c FROM (SELECT 1 AS a) AS x HAVING c > 0) SELECT c FROM y'
+
+    Args:
+        expression: Expression to pushdown.
+
+    Returns:
+        The expression with the CTE aliases pushed down into the projection.
+    """
+    for cte in expression.find_all(exp.CTE):
+        if cte.alias_column_names:
+            new_expressions = []
+            for _alias, projection in zip(cte.alias_column_names, cte.this.expressions):
+                if isinstance(projection, exp.Alias):
+                    projection.set("alias", _alias)
+                else:
+                    projection = alias(projection, alias=_alias)
+                new_expressions.append(projection)
+            cte.this.set("expressions", new_expressions)
+
+    return expression
+
+
 class Resolver:
     """
     Helper for resolving columns.
@@ -505,8 +580,8 @@ class Resolver:
     def __init__(self, scope: Scope, schema: Schema, infer_schema: bool = True):
         self.scope = scope
         self.schema = schema
-        self._source_columns: t.Optional[t.Dict[str, t.List[str]]] = None
-        self._unambiguous_columns: t.Optional[t.Dict[str, str]] = None
+        self._source_columns: t.Optional[t.Dict[str, t.Sequence[str]]] = None
+        self._unambiguous_columns: t.Optional[t.Mapping[str, str]] = None
         self._all_columns: t.Optional[t.Set[str]] = None
         self._infer_schema = infer_schema
 
@@ -559,7 +634,7 @@ class Resolver:
             }
         return self._all_columns
 
-    def get_source_columns(self, name: str, only_visible: bool = False) -> t.List[str]:
+    def get_source_columns(self, name: str, only_visible: bool = False) -> t.Sequence[str]:
         """Resolve the source columns for a given source `name`."""
         if name not in self.scope.sources:
             raise OptimizeError(f"Unknown table: {name}")
@@ -581,10 +656,15 @@ class Resolver:
         else:
             column_aliases = []
 
-        # If the source's columns are aliased, their aliases shadow the corresponding column names
-        return [alias or name for (name, alias) in itertools.zip_longest(columns, column_aliases)]
+        if column_aliases:
+            # If the source's columns are aliased, their aliases shadow the corresponding column names.
+            # This can be expensive if there are lots of columns, so only do this if column_aliases exist.
+            return [
+                alias or name for (name, alias) in itertools.zip_longest(columns, column_aliases)
+            ]
+        return columns
 
-    def _get_all_source_columns(self) -> t.Dict[str, t.List[str]]:
+    def _get_all_source_columns(self) -> t.Dict[str, t.Sequence[str]]:
         if self._source_columns is None:
             self._source_columns = {
                 source_name: self.get_source_columns(source_name)
@@ -595,8 +675,8 @@ class Resolver:
         return self._source_columns
 
     def _get_unambiguous_columns(
-        self, source_columns: t.Dict[str, t.List[str]]
-    ) -> t.Dict[str, str]:
+        self, source_columns: t.Dict[str, t.Sequence[str]]
+    ) -> t.Mapping[str, str]:
         """
         Find all the unambiguous columns in sources.
 
@@ -612,12 +692,17 @@ class Resolver:
         source_columns_pairs = list(source_columns.items())
 
         first_table, first_columns = source_columns_pairs[0]
-        unambiguous_columns = {col: first_table for col in self._find_unique_columns(first_columns)}
+
+        if len(source_columns_pairs) == 1:
+            # Performance optimization - avoid copying first_columns if there is only one table.
+            return SingleValuedMapping(first_columns, first_table)
+
+        unambiguous_columns = {col: first_table for col in first_columns}
         all_columns = set(unambiguous_columns)
 
         for table, columns in source_columns_pairs[1:]:
-            unique = self._find_unique_columns(columns)
-            ambiguous = set(all_columns).intersection(unique)
+            unique = set(columns)
+            ambiguous = all_columns.intersection(unique)
             all_columns.update(columns)
 
             for column in ambiguous:
@@ -626,19 +711,3 @@ class Resolver:
                 unambiguous_columns[column] = table
 
         return unambiguous_columns
-
-    @staticmethod
-    def _find_unique_columns(columns: t.Collection[str]) -> t.Set[str]:
-        """
-        Find the unique columns in a list of columns.
-
-        Example:
-            >>> sorted(Resolver._find_unique_columns(["a", "b", "b", "c"]))
-            ['a', 'c']
-
-        This is necessary because duplicate column names are ambiguous.
-        """
-        counts: t.Dict[str, int] = {}
-        for column in columns:
-            counts[column] = counts.get(column, 0) + 1
-        return {column for column, count in counts.items() if count == 1}
