@@ -26,6 +26,7 @@ class TestClickhouse(Validator):
         self.assertEqual(expr.sql(dialect="clickhouse"), "COUNT(x)")
         self.assertIsNone(expr._meta)
 
+        self.validate_identity("countIf(x, y)")
         self.validate_identity("x = y")
         self.validate_identity("x <> y")
         self.validate_identity("SELECT * FROM (SELECT a FROM b SAMPLE 0.01)")
@@ -41,12 +42,12 @@ class TestClickhouse(Validator):
         self.validate_identity("SELECT isNaN(1.0)")
         self.validate_identity("SELECT startsWith('Spider-Man', 'Spi')")
         self.validate_identity("SELECT xor(TRUE, FALSE)")
-        self.validate_identity("ATTACH DATABASE DEFAULT ENGINE = ORDINARY")
         self.validate_identity("CAST(['hello'], 'Array(Enum8(''hello'' = 1))')")
         self.validate_identity("SELECT x, COUNT() FROM y GROUP BY x WITH TOTALS")
-        self.validate_identity("SELECT INTERVAL t.days day")
+        self.validate_identity("SELECT INTERVAL t.days DAY")
         self.validate_identity("SELECT match('abc', '([a-z]+)')")
         self.validate_identity("dictGet(x, 'y')")
+        self.validate_identity("WITH final AS (SELECT 1) SELECT * FROM final")
         self.validate_identity("SELECT * FROM x FINAL")
         self.validate_identity("SELECT * FROM x AS y FINAL")
         self.validate_identity("'a' IN mapKeys(map('a', 1, 'b', 2))")
@@ -59,6 +60,9 @@ class TestClickhouse(Validator):
         self.validate_identity("SELECT * FROM foo GLOBAL LEFT ANY JOIN bla")
         self.validate_identity("SELECT quantile(0.5)(a)")
         self.validate_identity("SELECT quantiles(0.5)(a) AS x FROM t")
+        self.validate_identity("SELECT quantilesIf(0.5)(a, a > 1) AS x FROM t")
+        self.validate_identity("SELECT quantileState(0.5)(a) AS x FROM t")
+        self.validate_identity("SELECT deltaSumMerge(a) AS x FROM t")
         self.validate_identity("SELECT quantiles(0.1, 0.2, 0.3)(a)")
         self.validate_identity("SELECT quantileTiming(0.5)(RANGE(100))")
         self.validate_identity("SELECT histogram(5)(a)")
@@ -70,6 +74,26 @@ class TestClickhouse(Validator):
         self.validate_identity("CAST(x AS DATETIME)")
         self.validate_identity("CAST(x as MEDIUMINT)", "CAST(x AS Int32)")
         self.validate_identity("SELECT arrayJoin([1, 2, 3] AS src) AS dst, 'Hello', src")
+        self.validate_identity("""SELECT JSONExtractString('{"x": {"y": 1}}', 'x', 'y')""")
+        self.validate_identity("SELECT * FROM table LIMIT 1 BY a, b")
+        self.validate_identity("SELECT * FROM table LIMIT 2 OFFSET 1 BY a, b")
+        self.validate_identity(
+            "SELECT $1$foo$1$",
+            "SELECT 'foo'",
+        )
+        self.validate_identity(
+            "SELECT * FROM table LIMIT 1, 2 BY a, b",
+            "SELECT * FROM table LIMIT 2 OFFSET 1 BY a, b",
+        )
+        self.validate_identity(
+            "SELECT * FROM table LIMIT 1 BY CONCAT(datalayerVariantNo, datalayerProductId, warehouse)"
+        )
+        self.validate_identity(
+            """SELECT JSONExtractString('{"a": "hello", "b": [-100, 200.0, 300]}', 'a')"""
+        )
+        self.validate_identity(
+            "ATTACH DATABASE DEFAULT ENGINE = ORDINARY", check_command_warning=True
+        )
         self.validate_identity(
             "SELECT n, source FROM (SELECT toFloat32(number % 10) AS n, 'original' AS source FROM numbers(10) WHERE number % 3 = 1) ORDER BY n WITH FILL"
         )
@@ -131,7 +155,7 @@ class TestClickhouse(Validator):
             },
         )
         self.validate_all(
-            "SELECT CAST('2020-01-01' AS TIMESTAMP) + INTERVAL '500' microsecond",
+            "SELECT CAST('2020-01-01' AS TIMESTAMP) + INTERVAL '500' MICROSECOND",
             read={
                 "duckdb": "SELECT TIMESTAMP '2020-01-01' + INTERVAL '500 us'",
                 "postgres": "SELECT TIMESTAMP '2020-01-01' + INTERVAL '500 us'",
@@ -174,27 +198,27 @@ class TestClickhouse(Validator):
             },
         )
         self.validate_all(
-            "DATE_ADD('day', 1, x)",
+            "DATE_ADD(DAY, 1, x)",
             read={
-                "clickhouse": "dateAdd(day, 1, x)",
-                "presto": "DATE_ADD('day', 1, x)",
+                "clickhouse": "dateAdd(DAY, 1, x)",
+                "presto": "DATE_ADD('DAY', 1, x)",
             },
             write={
-                "clickhouse": "DATE_ADD('day', 1, x)",
-                "presto": "DATE_ADD('day', 1, x)",
-                "": "DATE_ADD(x, 1, 'day')",
+                "clickhouse": "DATE_ADD(DAY, 1, x)",
+                "presto": "DATE_ADD('DAY', 1, x)",
+                "": "DATE_ADD(x, 1, 'DAY')",
             },
         )
         self.validate_all(
-            "DATE_DIFF('day', a, b)",
+            "DATE_DIFF(DAY, a, b)",
             read={
-                "clickhouse": "dateDiff('day', a, b)",
-                "presto": "DATE_DIFF('day', a, b)",
+                "clickhouse": "dateDiff(DAY, a, b)",
+                "presto": "DATE_DIFF('DAY', a, b)",
             },
             write={
-                "clickhouse": "DATE_DIFF('day', a, b)",
-                "presto": "DATE_DIFF('day', a, b)",
-                "": "DATEDIFF(b, a, day)",
+                "clickhouse": "DATE_DIFF(DAY, a, b)",
+                "presto": "DATE_DIFF('DAY', a, b)",
+                "": "DATEDIFF(b, a, DAY)",
             },
         )
         self.validate_all(
@@ -345,6 +369,7 @@ class TestClickhouse(Validator):
             "SELECT STARTS_WITH('a', 'b'), STARTSWITH('a', 'b')",
             write={"clickhouse": "SELECT startsWith('a', 'b'), startsWith('a', 'b')"},
         )
+        self.validate_identity("SYSTEM STOP MERGES foo.bar", check_command_warning=True)
 
     def test_cte(self):
         self.validate_identity("WITH 'x' AS foo SELECT foo")
@@ -721,3 +746,21 @@ LIFETIME(MIN 0 MAX 0)""",
             },
             pretty=True,
         )
+        self.validate_identity("""CREATE TABLE ip_data (ip4 IPv4, ip6 IPv6) ENGINE=TinyLog()""")
+        self.validate_identity("""CREATE TABLE dates (dt1 Date32) ENGINE=TinyLog()""")
+        self.validate_all(
+            """
+            CREATE TABLE t (
+                a AggregateFunction(quantiles(0.5, 0.9), UInt64),
+                b AggregateFunction(quantiles, UInt64),
+                c SimpleAggregateFunction(sum, Float64)
+            )""",
+            write={
+                "clickhouse": """CREATE TABLE t (
+  a AggregateFunction(quantiles(0.5, 0.9), UInt64),
+  b AggregateFunction(quantiles, UInt64),
+  c SimpleAggregateFunction(sum, Float64)
+)"""
+            },
+            pretty=True,
+        )
