@@ -152,7 +152,7 @@ class TestHive(Validator):
                 "duckdb": "CREATE TABLE x (w TEXT)",  # Partition columns should exist in table
                 "presto": "CREATE TABLE x (w VARCHAR, y INTEGER, z INTEGER) WITH (PARTITIONED_BY=ARRAY['y', 'z'])",
                 "hive": "CREATE TABLE x (w STRING) PARTITIONED BY (y INT, z INT)",
-                "spark": "CREATE TABLE x (w STRING) PARTITIONED BY (y INT, z INT)",
+                "spark": "CREATE TABLE x (w STRING, y INT, z INT) PARTITIONED BY (y, z)",
             },
         )
         self.validate_all(
@@ -296,11 +296,11 @@ class TestHive(Validator):
         self.validate_all(
             "DATEDIFF(a, b)",
             write={
-                "duckdb": "DATE_DIFF('day', CAST(b AS DATE), CAST(a AS DATE))",
-                "presto": "DATE_DIFF('day', CAST(CAST(b AS TIMESTAMP) AS DATE), CAST(CAST(a AS TIMESTAMP) AS DATE))",
+                "duckdb": "DATE_DIFF('DAY', CAST(b AS DATE), CAST(a AS DATE))",
+                "presto": "DATE_DIFF('DAY', CAST(CAST(b AS TIMESTAMP) AS DATE), CAST(CAST(a AS TIMESTAMP) AS DATE))",
                 "hive": "DATEDIFF(TO_DATE(a), TO_DATE(b))",
                 "spark": "DATEDIFF(TO_DATE(a), TO_DATE(b))",
-                "": "DATEDIFF(TS_OR_DS_TO_DATE(a), TS_OR_DS_TO_DATE(b))",
+                "": "DATEDIFF(CAST(a AS DATE), CAST(b AS DATE))",
             },
         )
         self.validate_all(
@@ -358,11 +358,11 @@ class TestHive(Validator):
         self.validate_all(
             "DATEDIFF(TO_DATE(y), x)",
             write={
-                "duckdb": "DATE_DIFF('day', CAST(x AS DATE), CAST(CAST(y AS DATE) AS DATE))",
-                "presto": "DATE_DIFF('day', CAST(CAST(x AS TIMESTAMP) AS DATE), CAST(CAST(CAST(CAST(y AS TIMESTAMP) AS DATE) AS TIMESTAMP) AS DATE))",
+                "duckdb": "DATE_DIFF('DAY', CAST(x AS DATE), CAST(y AS DATE))",
+                "presto": "DATE_DIFF('DAY', CAST(CAST(x AS TIMESTAMP) AS DATE), CAST(CAST(CAST(CAST(y AS TIMESTAMP) AS DATE) AS TIMESTAMP) AS DATE))",
                 "hive": "DATEDIFF(TO_DATE(y), TO_DATE(x))",
                 "spark": "DATEDIFF(TO_DATE(y), TO_DATE(x))",
-                "": "DATEDIFF(TS_OR_DS_TO_DATE(TS_OR_DS_TO_DATE(y)), TS_OR_DS_TO_DATE(x))",
+                "": "DATEDIFF(CAST(y AS DATE), CAST(x AS DATE))",
             },
         )
         self.validate_all(
@@ -399,6 +399,8 @@ class TestHive(Validator):
         )
 
     def test_hive(self):
+        self.validate_identity("SET hiveconf:some_var = 5", check_command_warning=True)
+        self.validate_identity("(VALUES (1 AS a, 2 AS b, 3))")
         self.validate_identity("SELECT * FROM my_table TIMESTAMP AS OF DATE_ADD(CURRENT_DATE, -1)")
         self.validate_identity("SELECT * FROM my_table VERSION AS OF DATE_ADD(CURRENT_DATE, -1)")
 
@@ -439,13 +441,6 @@ class TestHive(Validator):
             "SELECT key, value, GROUPING__ID, COUNT(*) FROM T1 GROUP BY key, value WITH ROLLUP"
         )
 
-        self.validate_all(
-            "SET hiveconf:some_var = 5",
-            write={
-                "hive": "SET hiveconf:some_var = 5",
-                "spark": "SET hiveconf:some_var = 5",
-            },
-        )
         self.validate_all(
             "SELECT ${hiveconf:some_var}",
             write={
@@ -737,9 +732,11 @@ class TestHive(Validator):
             "SELECT * FROM x TABLESAMPLE (1 PERCENT) AS foo",
             read={
                 "presto": "SELECT * FROM x AS foo TABLESAMPLE BERNOULLI (1)",
+                "snowflake": "SELECT * FROM x AS foo TABLESAMPLE (1)",
             },
             write={
                 "hive": "SELECT * FROM x TABLESAMPLE (1 PERCENT) AS foo",
+                "snowflake": "SELECT * FROM x AS foo TABLESAMPLE (1)",
                 "spark": "SELECT * FROM x TABLESAMPLE (1 PERCENT) AS foo",
             },
         )
