@@ -182,12 +182,12 @@ class TestPresto(Validator):
     def test_interval_plural_to_singular(self):
         # Microseconds, weeks and quarters are not supported in Presto/Trino INTERVAL literals
         unit_to_expected = {
-            "SeCoNds": "second",
-            "minutes": "minute",
-            "hours": "hour",
-            "days": "day",
-            "months": "month",
-            "years": "year",
+            "SeCoNds": "SECOND",
+            "minutes": "MINUTE",
+            "hours": "HOUR",
+            "days": "DAY",
+            "months": "MONTH",
+            "years": "YEAR",
         }
 
         for unit, expected in unit_to_expected.items():
@@ -279,10 +279,10 @@ class TestPresto(Validator):
             },
         )
         self.validate_all(
-            "DATE_ADD('day', 1, x)",
+            "DATE_ADD('DAY', 1, x)",
             write={
-                "duckdb": "x + INTERVAL 1 day",
-                "presto": "DATE_ADD('day', 1, x)",
+                "duckdb": "x + INTERVAL 1 DAY",
+                "presto": "DATE_ADD('DAY', 1, x)",
                 "hive": "DATE_ADD(x, 1)",
                 "spark": "DATE_ADD(x, 1)",
             },
@@ -343,16 +343,16 @@ class TestPresto(Validator):
         )
 
         self.validate_all(
-            "SELECT timestamp '2012-10-31 00:00' AT TIME ZONE 'America/Sao_Paulo'",
+            "SELECT CAST('2012-10-31 00:00' AS TIMESTAMP) AT TIME ZONE 'America/Sao_Paulo'",
             write={
                 "spark": "SELECT FROM_UTC_TIMESTAMP(CAST('2012-10-31 00:00' AS TIMESTAMP), 'America/Sao_Paulo')",
-                "presto": "SELECT CAST('2012-10-31 00:00' AS TIMESTAMP) AT TIME ZONE 'America/Sao_Paulo'",
+                "presto": "SELECT AT_TIMEZONE(CAST('2012-10-31 00:00' AS TIMESTAMP), 'America/Sao_Paulo')",
             },
         )
         self.validate_all(
-            "CAST('2012-10-31 00:00' AS TIMESTAMP) AT TIME ZONE 'America/Sao_Paulo'",
+            "SELECT AT_TIMEZONE(CAST('2012-10-31 00:00' AS TIMESTAMP), 'America/Sao_Paulo')",
             read={
-                "spark": "FROM_UTC_TIMESTAMP('2012-10-31 00:00', 'America/Sao_Paulo')",
+                "spark": "SELECT FROM_UTC_TIMESTAMP(TIMESTAMP '2012-10-31 00:00', 'America/Sao_Paulo')",
             },
         )
         self.validate_all(
@@ -368,7 +368,7 @@ class TestPresto(Validator):
             "TIMESTAMP(x, 'America/Los_Angeles')",
             write={
                 "duckdb": "CAST(x AS TIMESTAMP) AT TIME ZONE 'America/Los_Angeles'",
-                "presto": "CAST(x AS TIMESTAMP) AT TIME ZONE 'America/Los_Angeles'",
+                "presto": "AT_TIMEZONE(CAST(x AS TIMESTAMP), 'America/Los_Angeles')",
             },
         )
         # this case isn't really correct, but it's a fall back for mysql's version
@@ -424,7 +424,7 @@ class TestPresto(Validator):
                 "duckdb": "CREATE TABLE x (w TEXT, y INT, z INT)",
                 "presto": "CREATE TABLE x (w VARCHAR, y INTEGER, z INTEGER) WITH (PARTITIONED_BY=ARRAY['y', 'z'])",
                 "hive": "CREATE TABLE x (w STRING) PARTITIONED BY (y INT, z INT)",
-                "spark": "CREATE TABLE x (w STRING) PARTITIONED BY (y INT, z INT)",
+                "spark": "CREATE TABLE x (w STRING, y INT, z INT) PARTITIONED BY (y, z)",
             },
         )
         self.validate_all(
@@ -546,17 +546,25 @@ class TestPresto(Validator):
 
     def test_unicode_string(self):
         for prefix in ("u&", "U&"):
-            self.validate_identity(
+            self.validate_all(
                 f"{prefix}'Hello winter \\2603 !'",
-                "U&'Hello winter \\2603 !'",
+                write={
+                    "presto": "U&'Hello winter \\2603 !'",
+                    "snowflake": "'Hello winter \\u2603 !'",
+                    "spark": "'Hello winter \\u2603 !'",
+                },
             )
-            self.validate_identity(
+            self.validate_all(
                 f"{prefix}'Hello winter #2603 !' UESCAPE '#'",
-                "U&'Hello winter #2603 !' UESCAPE '#'",
+                write={
+                    "presto": "U&'Hello winter #2603 !' UESCAPE '#'",
+                    "snowflake": "'Hello winter \\u2603 !'",
+                    "spark": "'Hello winter \\u2603 !'",
+                },
             )
 
     def test_presto(self):
-        with self.assertLogs(helper_logger) as cm:
+        with self.assertLogs(helper_logger):
             self.validate_all(
                 "SELECT COALESCE(ELEMENT_AT(MAP_FROM_ENTRIES(ARRAY[(51, '1')]), id), quantity) FROM my_table",
                 write={
@@ -574,6 +582,8 @@ class TestPresto(Validator):
                 },
             )
 
+        self.validate_identity("SELECT a FROM test TABLESAMPLE BERNOULLI (50)")
+        self.validate_identity("SELECT a FROM test TABLESAMPLE SYSTEM (75)")
         self.validate_identity("string_agg(x, ',')", "ARRAY_JOIN(ARRAY_AGG(x), ',')")
         self.validate_identity("SELECT * FROM x OFFSET 1 LIMIT 1")
         self.validate_identity("SELECT * FROM x OFFSET 1 FETCH FIRST 1 ROWS ONLY")
@@ -592,6 +602,16 @@ class TestPresto(Validator):
             "SELECT * FROM example.testdb.customer_orders FOR TIMESTAMP AS OF CAST('2022-03-23 09:59:29.803 Europe/Vienna' AS TIMESTAMP)"
         )
 
+        self.validate_all(
+            "SELECT LAST_DAY_OF_MONTH(CAST('2008-11-25' AS DATE))",
+            read={
+                "duckdb": "SELECT LAST_DAY(CAST('2008-11-25' AS DATE))",
+            },
+            write={
+                "duckdb": "SELECT LAST_DAY(CAST('2008-11-25' AS DATE))",
+                "presto": "SELECT LAST_DAY_OF_MONTH(CAST('2008-11-25' AS DATE))",
+            },
+        )
         self.validate_all(
             "SELECT MAX_BY(a.id, a.timestamp) FROM a",
             read={
@@ -699,9 +719,9 @@ class TestPresto(Validator):
             },
         )
         self.validate_all("VALUES 1, 2, 3", write={"presto": "VALUES (1), (2), (3)"})
-        self.validate_all("INTERVAL '1 day'", write={"trino": "INTERVAL '1' day"})
-        self.validate_all("(5 * INTERVAL '7' day)", read={"": "INTERVAL '5' week"})
-        self.validate_all("(5 * INTERVAL '7' day)", read={"": "INTERVAL '5' WEEKS"})
+        self.validate_all("INTERVAL '1 day'", write={"trino": "INTERVAL '1' DAY"})
+        self.validate_all("(5 * INTERVAL '7' DAY)", read={"": "INTERVAL '5' WEEK"})
+        self.validate_all("(5 * INTERVAL '7' DAY)", read={"": "INTERVAL '5' WEEKS"})
         self.validate_all(
             "SELECT SUBSTRING(a, 1, 3), SUBSTRING(a, LENGTH(a) - (3 - 1))",
             read={
