@@ -76,6 +76,7 @@ map.x
 SELECT update
 SELECT x.update
 SELECT call.x
+SELECT end
 a.b.INT(1.234)
 INT(x / 100)
 time * 100
@@ -86,6 +87,7 @@ x IN ('a', 'a''a')
 x IN ((1))
 x BETWEEN -1 AND 1
 x BETWEEN 'a' || b AND 'c' || d
+((a, b) AS c)
 NOT x IS NULL
 x IS TRUE
 x IS FALSE
@@ -105,6 +107,7 @@ CURRENT_DATE('UTC')
 CURRENT_DATE AT TIME ZONE 'UTC'
 CURRENT_DATE AT TIME ZONE zone_column
 CURRENT_DATE AT TIME ZONE 'UTC' AT TIME ZONE 'Asia/Tokio'
+f1 AT TIME ZONE INTERVAL '-10:00' AS f1
 ARRAY()
 ARRAY(1, 2)
 ARRAY(time, foo)
@@ -135,19 +138,27 @@ x ILIKE '%y%'
 x LIKE '%y%' ESCAPE '\'
 x ILIKE '%y%' ESCAPE '\'
 1 AS escape
-INTERVAL '1' day
+INTERVAL '1' DAY
 INTERVAL '1' MONTH
+INTERVAL '1' YEAR
 INTERVAL '-1' CURRENT_DATE
 INTERVAL '-31' CAST(GETDATE() AS DATE)
 INTERVAL (1 + 3) DAYS
-INTERVAL '1' day * 5
-5 * INTERVAL '1' day
-CASE WHEN TRUE THEN INTERVAL '15' days END
+INTERVAL '1' DAY * 5
+5 * INTERVAL '1' DAY
+CASE WHEN TRUE THEN INTERVAL '15' DAYS END
+CASE WHEN TRUE THEN 1 ELSE interval END
+CASE WHEN TRUE THEN 1 ELSE "INTERVAL" END
+SELECT * WHERE interval IS NULL
+SELECT * WHERE NOT interval IS NULL
+SELECT * WHERE INTERVAL "is" > 1
+SELECT * WHERE INTERVAL x.is > 1
 CAST('45' AS INTERVAL DAYS)
 CAST(x AS UUID)
 FILTER(a, x -> x.a.b.c.d.e.f.g)
 FILTER(a, x -> FOO(x.a.b.c.d.e.f.g) + x.a.b.c.d.e.f.g)
-TIMESTAMP_DIFF(CURRENT_TIMESTAMP(), 1, DAY)
+TIMESTAMP_FROM_PARTS(2019, 1, 10, 2, 3, 4, 123456789, 'America/Los_Angeles')
+TIMESTAMPDIFF(CURRENT_TIMESTAMP(), 1, DAY)
 DATETIME_DIFF(CURRENT_DATE, 1, DAY)
 QUANTILE(x, 0.5)
 REGEXP_REPLACE('new york', '(\w)(\w*)', x -> UPPER(x[1]) || LOWER(x[2]))
@@ -161,7 +172,7 @@ REPLACE(1)
 DATE(x) = DATE(y)
 TIMESTAMP(DATE(x))
 TIMESTAMP_TRUNC(COALESCE(time_field, CURRENT_TIMESTAMP()), DAY)
-COUNT(DISTINCT CASE WHEN DATE_TRUNC(DATE(time_field), isoweek) = DATE_TRUNC(DATE(time_field2), isoweek) THEN report_id ELSE NULL END)
+COUNT(DISTINCT CASE WHEN DATE_TRUNC('ISOWEEK', DATE(time_field)) = DATE_TRUNC('ISOWEEK', DATE(time_field2)) THEN report_id ELSE NULL END)
 COUNT(a, b)
 x[y - 1]
 CASE WHEN SUM(x) > 3 THEN 1 END OVER (PARTITION BY x)
@@ -177,12 +188,10 @@ CASE 1 WHEN 1 THEN timestamp ELSE date END
 x AT TIME ZONE 'UTC'
 CAST('2025-11-20 00:00:00+00' AS TIMESTAMP) AT TIME ZONE 'Africa/Cairo'
 SET x = 1
-SET -v
 SET x = ';'
 SET variable = value
 SET GLOBAL variable = value
 SET LOCAL variable = value
-SET @user OFF
 @x
 @"x"
 COMMIT
@@ -221,20 +230,15 @@ SELECT DISTINCT ON (x, y + 1) * FROM z
 SELECT DISTINCT ON (x.y) * FROM z
 SELECT DISTINCT FROM_SOMETHING
 SELECT top.x
-SELECT TIMESTAMP(DATE_TRUNC(DATE(time_field), MONTH)) AS time_value FROM "table"
+SELECT TIMESTAMP(DATE_TRUNC('MONTH', DATE(time_field))) AS time_value FROM "table"
 SELECT GREATEST((3 + 1), LEAST(3, 4))
 SELECT TRANSFORM(a, b -> b) AS x
 SELECT AGGREGATE(a, (a, b) -> a + b) AS x
 SELECT COUNT(DISTINCT a, b)
 SELECT COUNT(DISTINCT a, b + 1)
 SELECT SUM(DISTINCT x)
-SELECT SUM(x IGNORE NULLS) AS x
-SELECT COUNT(x RESPECT NULLS)
 SELECT TRUNCATE(a, b)
-SELECT ARRAY_AGG(DISTINCT x IGNORE NULLS ORDER BY a, b DESC LIMIT 10) AS x
-SELECT ARRAY_AGG(DISTINCT x IGNORE NULLS ORDER BY a, b DESC LIMIT 1, 10) AS x
 SELECT ARRAY_AGG(STRUCT(x, x AS y) ORDER BY z DESC) AS x
-SELECT LAST_VALUE(x IGNORE NULLS) OVER y AS x
 SELECT LAG(x) OVER (ORDER BY y) AS x
 SELECT LEAD(a) OVER (ORDER BY b) AS a
 SELECT LEAD(a, 1) OVER (PARTITION BY a ORDER BY a) AS x
@@ -269,6 +273,8 @@ SELECT x LIKE '%x%' FROM test
 SELECT * FROM test LIMIT 100
 SELECT * FROM test LIMIT 1 + 1
 SELECT * FROM test LIMIT 100 OFFSET 200
+SELECT * FROM test LIMIT (SELECT 1)
+SELECT * FROM test LIMIT (SELECT 1) OFFSET (SELECT 1)
 SELECT * FROM test FETCH FIRST ROWS ONLY
 SELECT * FROM test FETCH FIRST 1 ROWS ONLY
 SELECT * FROM test ORDER BY id DESC FETCH FIRST 10 ROWS WITH TIES
@@ -330,17 +336,13 @@ SELECT a FROM test TABLESAMPLE (BUCKET 1 OUT OF 5)
 SELECT a FROM test TABLESAMPLE (BUCKET 1 OUT OF 5 ON x)
 SELECT a FROM test TABLESAMPLE (BUCKET 1 OUT OF 5 ON RAND())
 SELECT a FROM test TABLESAMPLE (0.1 PERCENT)
-SELECT a FROM test TABLESAMPLE (100)
 SELECT a FROM test TABLESAMPLE (100 ROWS)
-SELECT a FROM test TABLESAMPLE BERNOULLI (50)
-SELECT a FROM test TABLESAMPLE SYSTEM (75)
 SELECT a FROM test PIVOT(SUM(x) FOR y IN ('z', 'q'))
 SELECT 1 FROM a.b.table1 AS t UNPIVOT((c3) FOR c4 IN (a, b))
 SELECT a FROM test PIVOT(SOMEAGG(x, y, z) FOR q IN (1))
 SELECT a FROM test PIVOT(SUM(x) FOR y IN ('z', 'q')) PIVOT(MAX(b) FOR c IN ('d'))
 SELECT a FROM (SELECT a, b FROM test) PIVOT(SUM(x) FOR y IN ('z', 'q'))
 SELECT a FROM test UNPIVOT(x FOR y IN (z, q)) AS x
-SELECT a FROM test PIVOT(SUM(x) FOR y IN ('z', 'q')) AS x TABLESAMPLE (0.1)
 SELECT a FROM test PIVOT(SUM(x) FOR y IN ('z', 'q')) UNPIVOT(x FOR y IN (z, q)) AS x
 SELECT ABS(a) FROM test
 SELECT AVG(a) FROM test
@@ -354,9 +356,10 @@ SELECT COUNT(DISTINCT a) FROM test
 SELECT EXP(a) FROM test
 SELECT FLOOR(a) FROM test
 SELECT FLOOR(a, b) FROM test
-SELECT FIRST(a) FROM test
+SELECT FIRST_VALUE(a) FROM test
 SELECT GREATEST(a, b, c) FROM test
-SELECT LAST(a) FROM test
+SELECT LAST_VALUE(a) FROM test
+SELECT LAST_VALUE(a) IGNORE NULLS OVER () + 1
 SELECT LN(a) FROM test
 SELECT LOG10(a) FROM test
 SELECT MAX(a) FROM test
@@ -396,6 +399,7 @@ SELECT 1 FROM a INNER JOIN b ON a.x = b.x
 SELECT 1 FROM a LEFT JOIN b ON a.x = b.x
 SELECT 1 FROM a RIGHT JOIN b ON a.x = b.x
 SELECT 1 FROM a CROSS JOIN b ON a.x = b.x
+SELECT 1 FROM a SEMI JOIN b ON a.x = b.x
 SELECT 1 FROM a LEFT SEMI JOIN b ON a.x = b.x
 SELECT 1 FROM a LEFT ANTI JOIN b ON a.x = b.x
 SELECT 1 FROM a RIGHT SEMI JOIN b ON a.x = b.x
@@ -423,6 +427,8 @@ SELECT 1 INTERSECT SELECT 2
 SELECT 1 INTERSECT SELECT 2
 SELECT 1 AS delete, 2 AS alter
 SELECT * FROM (x)
+SELECT * FROM ((x))
+SELECT * FROM (((x)))
 SELECT * FROM ((SELECT 1))
 SELECT * FROM (x CROSS JOIN foo LATERAL VIEW EXPLODE(y))
 SELECT * FROM (SELECT 1) AS x
@@ -432,6 +438,14 @@ SELECT * FROM (SELECT 1 UNION ALL SELECT 2)
 SELECT * FROM ((SELECT 1) AS a UNION ALL (SELECT 2) AS b)
 SELECT * FROM ((SELECT 1) AS a(b))
 SELECT * FROM ((SELECT 1) UNION (SELECT 2) UNION (SELECT 3))
+SELECT * FROM (table1 AS t1 LEFT JOIN table2 AS t2 ON 1 = 1)
+SELECT * FROM (tbl1 LEFT JOIN tbl2 ON 1 = 1)
+SELECT * FROM (tbl1, tbl2 JOIN tbl3 ON TRUE)
+SELECT * FROM (tbl1 CROSS JOIN tbl2)
+SELECT * FROM (tbl1 CROSS JOIN tbl2) AS t
+SELECT * FROM (tbl AS tbl) AS t
+SELECT * FROM (tbl1 JOIN (tbl2 CROSS JOIN tbl3) ON bla = foo)
+SELECT * FROM (tbl1, LATERAL (SELECT * FROM bla) AS tbl)
 SELECT * FROM x AS y(a, b)
 SELECT * EXCEPT (a, b)
 SELECT * EXCEPT (a, b) FROM y
@@ -574,12 +588,12 @@ CREATE TABLE IF NOT EXISTS customer (pk BIGINT NOT NULL GENERATED ALWAYS AS IDEN
 CREATE TABLE customer (pk BIGINT NOT NULL GENERATED ALWAYS AS IDENTITY (START WITH 10 INCREMENT BY 1 MINVALUE -1 MAXVALUE 1 NO CYCLE))
 CREATE TABLE customer (pk BIGINT NOT NULL GENERATED ALWAYS AS IDENTITY (START WITH 10))
 CREATE TABLE customer (pk BIGINT NOT NULL GENERATED ALWAYS AS IDENTITY (CYCLE))
+CREATE TABLE customer (period INT NOT NULL)
 CREATE TABLE foo (baz_id INT REFERENCES baz (id) DEFERRABLE)
 CREATE TABLE foo (baz CHAR(4) CHARACTER SET LATIN UPPERCASE NOT CASESPECIFIC COMPRESS 'a')
 CREATE TABLE foo (baz DATE FORMAT 'YYYY/MM/DD' TITLE 'title' INLINE LENGTH 1 COMPRESS ('a', 'b'))
 CREATE TABLE t (title TEXT)
 CREATE TABLE foo (baz INT, inline TEXT)
-CREATE SET GLOBAL TEMPORARY TABLE a, NO BEFORE JOURNAL, NO AFTER JOURNAL, MINIMUM DATABLOCKSIZE, BLOCKCOMPRESSION=NEVER (a INT)
 CREATE ALGORITHM=UNDEFINED DEFINER=foo@% SQL SECURITY DEFINER VIEW a AS (SELECT a FROM b)
 CREATE TEMPORARY TABLE x AS SELECT a FROM d
 CREATE TEMPORARY TABLE IF NOT EXISTS x AS SELECT a FROM d
@@ -605,19 +619,23 @@ CREATE FUNCTION a(b INT, c VARCHAR) AS 'SELECT 1'
 CREATE FUNCTION a() LANGUAGE sql
 CREATE FUNCTION a() LANGUAGE sql RETURNS INT
 CREATE FUNCTION a.b(x INT) RETURNS INT AS RETURN x + 1
+CREATE FUNCTION a.b(x TEXT) RETURNS TEXT CONTAINS SQL AS RETURN x
+CREATE FUNCTION a.b(x TEXT) RETURNS TEXT LANGUAGE SQL MODIFIES SQL DATA AS RETURN x
+CREATE FUNCTION a.b(x TEXT) LANGUAGE SQL READS SQL DATA RETURNS TEXT AS RETURN x
 CREATE FUNCTION a.b.c()
-CREATE INDEX abc ON t (a)
-CREATE INDEX abc ON t (a, b, b)
-CREATE INDEX abc ON t (a NULLS LAST)
+CREATE INDEX abc ON t(a)
+CREATE INDEX "abc" ON t(a)
+CREATE INDEX abc ON t(a, b, b)
+CREATE INDEX abc ON t(a NULLS LAST)
 CREATE INDEX pointloc ON points USING GIST(BOX(location, location))
-CREATE UNIQUE INDEX abc ON t (a, b, b)
-CREATE UNIQUE INDEX IF NOT EXISTS my_idx ON tbl (a, b)
+CREATE UNIQUE INDEX abc ON t(a, b, b)
+CREATE UNIQUE INDEX IF NOT EXISTS my_idx ON tbl(a, b)
 CREATE SCHEMA x
 CREATE SCHEMA IF NOT EXISTS y
 CREATE DATABASE x
 CREATE DATABASE IF NOT EXISTS y
 CREATE PROCEDURE IF NOT EXISTS a.b.c() AS 'DECLARE BEGIN; END'
-CREATE OR REPLACE STAGE
+CREATE TABLE T3 AS (SELECT DISTINCT A FROM T1 EXCEPT (SELECT A FROM T2) LIMIT 1)
 DESCRIBE x
 DROP INDEX a.b.c
 DROP FUNCTION a.b.c (INT)
@@ -630,30 +648,17 @@ CACHE LAZY TABLE x OPTIONS('storageLevel' = 'value') AS WITH a AS (SELECT 1) SEL
 CACHE LAZY TABLE x AS WITH a AS (SELECT 1) SELECT a.* FROM a
 CACHE TABLE x AS WITH a AS (SELECT 1) SELECT a.* FROM a
 CACHE TABLE x AS (SELECT 1 AS y)
-CALL catalog.system.iceberg_procedure_name(named_arg_1 => 'arg_1', named_arg_2 => 'arg_2')
 DROP PROCEDURE a.b.c (INT)
 INSERT OVERWRITE TABLE a.b PARTITION(ds) SELECT x FROM y
 INSERT OVERWRITE TABLE a.b PARTITION(ds = 'YYYY-MM-DD') SELECT x FROM y
 INSERT OVERWRITE TABLE a.b PARTITION(ds, hour) SELECT x FROM y
 INSERT OVERWRITE TABLE a.b PARTITION(ds = 'YYYY-MM-DD', hour = 'hh') SELECT x FROM y
-ALTER SESSION SET STATEMENT_TIMEOUT_IN_SECONDS=3
-ALTER AGGREGATE bla(foo) OWNER TO CURRENT_USER
-ALTER RULE foo ON bla RENAME TO baz
-ALTER ROLE CURRENT_USER WITH REPLICATION
-ALTER SEQUENCE IF EXISTS baz RESTART WITH boo
-ALTER TYPE electronic_mail RENAME TO email
-ALTER VIEW foo ALTER COLUMN bla SET DEFAULT 'NOT SET'
-ALTER DOMAIN foo VALIDATE CONSTRAINT bla
-ALTER schema doo
-ANALYZE a.y
 DELETE FROM x WHERE y > 1
 DELETE FROM y
 DELETE FROM event USING sales WHERE event.eventid = sales.eventid
 DELETE FROM event USING sales, bla WHERE event.eventid = sales.eventid
 DELETE FROM event USING sales AS s WHERE event.eventid = s.eventid
 DELETE FROM event AS event USING sales AS s WHERE event.eventid = s.eventid
-PREPARE statement
-EXECUTE statement
 DROP TABLE a
 DROP TABLE a.b
 DROP TABLE IF EXISTS a
@@ -665,12 +670,10 @@ DROP VIEW a
 DROP VIEW a.b
 DROP VIEW IF EXISTS a
 DROP VIEW IF EXISTS a.b
-SHOW TABLES
 USE db
 BEGIN
 ROLLBACK
 ROLLBACK TO b
-EXPLAIN SELECT * FROM x
 INSERT INTO x SELECT * FROM y
 INSERT INTO x (SELECT * FROM y)
 INSERT INTO x WITH y AS (SELECT 1) SELECT * FROM y
@@ -686,7 +689,6 @@ INSERT OVERWRITE TABLE a.b IF EXISTS SELECT * FROM y
 INSERT OVERWRITE DIRECTORY 'x' SELECT 1
 INSERT OVERWRITE LOCAL DIRECTORY 'x' SELECT 1
 INSERT OVERWRITE LOCAL DIRECTORY 'x' ROW FORMAT DELIMITED FIELDS TERMINATED BY '1' COLLECTION ITEMS TERMINATED BY '2' MAP KEYS TERMINATED BY '3' LINES TERMINATED BY '4' NULL DEFINED AS '5' SELECT 1
-LOAD foo
 LOAD DATA INPATH 'x' INTO TABLE y PARTITION(ds = 'yyyy')
 LOAD DATA LOCAL INPATH 'x' INTO TABLE y PARTITION(ds = 'yyyy')
 LOAD DATA LOCAL INPATH 'x' INTO TABLE y PARTITION(ds = 'yyyy') INPUTFORMAT 'y'
@@ -700,10 +702,8 @@ UPDATE tbl_name SET foo = 123
 UPDATE tbl_name SET foo = 123, bar = 345
 UPDATE db.tbl_name SET foo = 123 WHERE tbl_name.bar = 234
 UPDATE db.tbl_name SET foo = 123, foo_1 = 234 WHERE tbl_name.bar = 234
-TRUNCATE TABLE x
-OPTIMIZE TABLE y
-VACUUM FREEZE my_table
-COMMENT ON ACCESS METHOD gin IS 'GIN index access method'
+UPDATE products SET price = price * 1.10 WHERE price <= 99.99 RETURNING name, price AS new_price
+UPDATE t1 AS a, t2 AS b, t3 AS c LEFT JOIN t4 AS d ON c.id = d.id SET a.id = 1
 COMMENT ON COLUMN my_schema.my_table.my_column IS 'Employee ID number'
 COMMENT ON DATABASE my_database IS 'Development Database'
 COMMENT ON PROCEDURE my_proc(integer, integer) IS 'Runs a report'
@@ -712,21 +712,17 @@ WITH a AS (SELECT 1) INSERT INTO b SELECT * FROM a
 WITH a AS (SELECT * FROM b) UPDATE a SET col = 1
 WITH a AS (SELECT * FROM b) CREATE TABLE b AS SELECT * FROM a
 WITH a AS (SELECT * FROM b) DELETE FROM a
-WITH a AS (SELECT * FROM b) CACHE TABLE a
 SELECT ? AS ? FROM x WHERE b BETWEEN ? AND ? GROUP BY ?, 1 LIMIT ?
 SELECT :hello, ? FROM x LIMIT :my_limit
+SELECT a FROM b WHERE c IS ?
 SELECT * FROM x OFFSET @skip FETCH NEXT @take ROWS ONLY
 WITH a AS ((SELECT b.foo AS foo, b.bar AS bar FROM b) UNION ALL (SELECT c.foo AS foo, c.bar AS bar FROM c)) SELECT * FROM a
 WITH a AS ((SELECT 1 AS b) UNION ALL (SELECT 1 AS b)) SELECT * FROM a
 SELECT (WITH x AS (SELECT 1 AS y) SELECT * FROM x) AS z
 SELECT ((SELECT 1) + 1)
 SELECT * FROM project.dataset.INFORMATION_SCHEMA.TABLES
-SELECT * FROM (table1 AS t1 LEFT JOIN table2 AS t2 ON 1 = 1)
-SELECT * FROM (tbl1 LEFT JOIN tbl2 ON 1 = 1)
-SELECT * FROM (tbl1, tbl2 JOIN tbl3 ON TRUE)
-SELECT * FROM (tbl1 JOIN (tbl2 CROSS JOIN tbl3) ON bla = foo)
-SELECT * FROM (tbl1, LATERAL (SELECT * FROM bla) AS tbl)
 SELECT CAST(x AS INT) /* comment */ FROM foo
+SELECT c /* c1 */ AS alias /* c2 */
 SELECT a /* x */, b /* x */
 SELECT a /* x */ /* y */ /* z */, b /* k */ /* m */
 SELECT * FROM foo /* x */, bla /* x */
@@ -742,9 +738,6 @@ INSERT INTO foo SELECT * FROM bar /* comment */
 /* c */ WITH x AS (SELECT 1) SELECT * FROM x
 SELECT a FROM x WHERE a COLLATE 'utf8_general_ci' = 'b'
 SELECT x AS INTO FROM bla
-SELECT * INTO newevent FROM event
-SELECT * INTO TEMPORARY newevent FROM event
-SELECT * INTO UNLOGGED newevent FROM event
 ALTER TABLE integers ADD COLUMN k INT
 ALTER TABLE integers ADD COLUMN k INT FIRST
 ALTER TABLE integers ADD COLUMN k INT AFTER m
@@ -753,20 +746,21 @@ ALTER TABLE IF EXISTS integers ADD COLUMN k INT
 ALTER TABLE integers ADD COLUMN l INT DEFAULT 10
 ALTER TABLE measurements ADD COLUMN mtime TIMESTAMPTZ DEFAULT NOW()
 ALTER TABLE integers DROP COLUMN k
-ALTER TABLE integers DROP PRIMARY KEY
 ALTER TABLE integers DROP COLUMN IF EXISTS k
 ALTER TABLE integers DROP COLUMN k CASCADE
-ALTER TABLE integers ALTER COLUMN i TYPE VARCHAR
-ALTER TABLE integers ALTER COLUMN i TYPE VARCHAR USING CONCAT(i, '_', j)
+ALTER TABLE integers ALTER COLUMN i SET DATA TYPE VARCHAR
+ALTER TABLE integers ALTER COLUMN i SET DATA TYPE VARCHAR USING CONCAT(i, '_', j)
 ALTER TABLE integers ALTER COLUMN i SET DEFAULT 10
 ALTER TABLE integers ALTER COLUMN i DROP DEFAULT
+ALTER TABLE ingredients ALTER COLUMN amount COMMENT 'tablespoons'
 ALTER TABLE mydataset.mytable DROP COLUMN A, DROP COLUMN IF EXISTS B
 ALTER TABLE mydataset.mytable ADD COLUMN A TEXT, ADD COLUMN IF NOT EXISTS B INT
 ALTER TABLE orders DROP PARTITION(dt = '2014-05-14', country = 'IN')
 ALTER TABLE orders DROP IF EXISTS PARTITION(dt = '2014-05-14', country = 'IN')
 ALTER TABLE orders DROP PARTITION(dt = '2014-05-14', country = 'IN'), PARTITION(dt = '2014-05-15', country = 'IN')
 ALTER TABLE mydataset.mytable DELETE WHERE x = 1
-ALTER TABLE table1 MODIFY COLUMN name1 SET TAG foo='bar'
+ALTER TABLE table1 RENAME COLUMN c1 TO c2
+ALTER TABLE table1 RENAME COLUMN IF EXISTS c1 TO c2
 SELECT div.a FROM test_table AS div
 WITH view AS (SELECT 1 AS x) SELECT * FROM view
 ARRAY<STRUCT<INT, DOUBLE, ARRAY<INT>>>
@@ -786,12 +780,12 @@ ALTER TABLE pets ADD CONSTRAINT pets_persons_fk FOREIGN KEY (owner_first_name, o
 ALTER TABLE pets ADD CONSTRAINT pets_name_not_cute_chk CHECK (LENGTH(name) < 20)
 ALTER TABLE people10m ADD CONSTRAINT dateWithinRange CHECK (birthDate > '1900-01-01')
 ALTER TABLE people10m ADD CONSTRAINT validIds CHECK (id > 1 AND id < 99999999) ENFORCED
-ALTER TABLE s_ut ADD CONSTRAINT s_ut_uq UNIQUE hajo
 ALTER TABLE baa ADD CONSTRAINT boo PRIMARY KEY (x, y) NOT ENFORCED DEFERRABLE INITIALLY DEFERRED NORELY
 ALTER TABLE baa ADD CONSTRAINT boo PRIMARY KEY (x, y) NOT ENFORCED DEFERRABLE INITIALLY DEFERRED NORELY
 ALTER TABLE baa ADD CONSTRAINT boo FOREIGN KEY (x, y) REFERENCES persons ON UPDATE NO ACTION ON DELETE NO ACTION MATCH FULL
 ALTER TABLE a ADD PRIMARY KEY (x, y) NOT ENFORCED
 ALTER TABLE a ADD FOREIGN KEY (x, y) REFERENCES bla
+ALTER TABLE s_ut ADD CONSTRAINT s_ut_uq UNIQUE hajo
 SELECT partition FROM a
 SELECT end FROM a
 SELECT id FROM b.a AS a QUALIFY ROW_NUMBER() OVER (PARTITION BY br ORDER BY sadf DESC) = 1
@@ -802,7 +796,6 @@ SELECT LEFT FROM LEFT LEFT JOIN RIGHT RIGHT JOIN LEFT
 SELECT * FROM x WHERE name ILIKE ANY XXX('a', 'b')
 SELECT * FROM x WHERE name LIKE ANY XXX('a', 'b')
 a OVERLAPS b
-GRANT INSERT ON foo TO bla
 PRAGMA quick_check
 PRAGMA QUICK_CHECK(0)
 PRAGMA QUICK_CHECK('sqlite_master')
@@ -824,12 +817,11 @@ JSON_OBJECT('x': NULL, 'y': 1 WITH UNIQUE KEYS)
 JSON_OBJECT('x': NULL, 'y': 1 ABSENT ON NULL WITH UNIQUE KEYS)
 JSON_OBJECT('x': 1 RETURNING VARCHAR(100))
 JSON_OBJECT('x': 1 RETURNING VARBINARY FORMAT JSON ENCODING UTF8)
+PRIOR AS x
 SELECT if.x
 SELECT NEXT VALUE FOR db.schema.sequence_name
 SELECT NEXT VALUE FOR db.schema.sequence_name OVER (ORDER BY foo), col
 SELECT PERCENTILE_CONT(x, 0.5) OVER ()
-SELECT PERCENTILE_CONT(x, 0.5 RESPECT NULLS) OVER ()
-SELECT PERCENTILE_CONT(x, 0.5 IGNORE NULLS) OVER ()
 WITH my_cte AS (SELECT 'a' AS desc) SELECT desc AS description FROM my_cte
 WITH my_cte AS (SELECT 'a' AS asc) SELECT asc AS description FROM my_cte
 SELECT * FROM case
@@ -838,3 +830,24 @@ SELECT * FROM current_date
 SELECT * FROM schema.current_date
 SELECT /*+ SOME_HINT(foo) */ 1
 SELECT * FROM (tbl1 CROSS JOIN (SELECT * FROM tbl2) AS t1)
+/* comment1 */ INSERT INTO x /* comment2 */ VALUES (1, 2, 3)
+/* comment1 */ UPDATE tbl /* comment2 */ SET x = 2 WHERE x < 2
+/* comment1 */ DELETE FROM x /* comment2 */ WHERE y > 1
+/* comment */ CREATE TABLE foo AS SELECT 1
+SELECT next, transform, if
+SELECT "any", "case", "if", "next"
+SELECT x FROM y ORDER BY x ASC
+KILL '123'
+KILL CONNECTION 123
+KILL QUERY '123'
+CHR(97)
+SELECT * FROM UNNEST(x) WITH ORDINALITY UNION ALL SELECT * FROM UNNEST(y) WITH ORDINALITY
+SELECT x FROM t1 UNION ALL SELECT x FROM t2 LIMIT 1
+SELECT x FROM t1 UNION ALL SELECT x FROM t2 UNION ALL SELECT x FROM t3 LIMIT 1
+WITH use(use) AS (SELECT 1) SELECT use FROM use
+SELECT recursive FROM t
+SELECT (ROW_NUMBER() OVER (PARTITION BY user ORDER BY date ASC) - ROW_NUMBER() OVER (PARTITION BY user, segment ORDER BY date ASC)) AS group_id FROM example_table
+CAST(foo AS BPCHAR)
+values
+SELECT values
+SELECT values AS values FROM t WHERE values + 1 > 3
