@@ -86,6 +86,8 @@ class TestBuild(unittest.TestCase):
                 "CASE x WHEN 1 THEN x ELSE bar END",
             ),
             (lambda: exp.func("COALESCE", "x", 1), "COALESCE(x, 1)"),
+            (lambda: exp.column("x").desc(), "x DESC"),
+            (lambda: exp.column("x").desc(nulls_first=True), "x DESC NULLS FIRST"),
             (lambda: select("x"), "SELECT x"),
             (lambda: select("x"), "SELECT x"),
             (lambda: select("x", "y"), "SELECT x, y"),
@@ -630,6 +632,10 @@ class TestBuild(unittest.TestCase):
                 lambda: exp.insert("VALUES (1, 2), (3, 4)", "tbl", columns=["cola", "colb"]),
                 "INSERT INTO tbl (cola, colb) VALUES (1, 2), (3, 4)",
             ),
+            (
+                lambda: exp.insert("VALUES (1), (2)", "tbl", columns=["col a"]),
+                'INSERT INTO tbl ("col a") VALUES (1), (2)',
+            ),
             (
                 lambda: exp.insert("SELECT * FROM cte", "t").with_("cte", as_="SELECT x FROM tbl"),
                 "WITH cte AS (SELECT x FROM tbl) INSERT INTO t SELECT * FROM cte",
@@ -647,6 +653,18 @@ class TestBuild(unittest.TestCase):
                 lambda: exp.cast_unless("CAST(x AS TEXT)", "int", "int"),
                 "CAST(CAST(x AS TEXT) AS INT)",
             ),
+            (
+                lambda: exp.rename_column("table1", "c1", "c2", True),
+                "ALTER TABLE table1 RENAME COLUMN IF EXISTS c1 TO c2",
+            ),
+            (
+                lambda: exp.rename_column("table1", "c1", "c2", False),
+                "ALTER TABLE table1 RENAME COLUMN c1 TO c2",
+            ),
+            (
+                lambda: exp.rename_column("table1", "c1", "c2"),
+                "ALTER TABLE table1 RENAME COLUMN c1 TO c2",
+            ),
         ]:
             with self.subTest(sql):
                 self.assertEqual(expression().sql(dialect[0] if dialect else None), sql)
