@@ -217,6 +217,8 @@ class TestExpressions(unittest.TestCase):
             "foo.`{bar,er}`",
         )
 
+        self.assertEqual(exp.table_name(bq_dashed_table, identify=True), '"a-1"."b"."c"')
+
     def test_table(self):
         self.assertEqual(exp.table_("a", alias="b"), parse_one("select * from a b").find(exp.Table))
         self.assertEqual(exp.table_("a", "").sql(), "a")
@@ -238,7 +240,16 @@ class TestExpressions(unittest.TestCase):
                 },
                 dialect="bigquery",
             ).sql(),
-            'SELECT * FROM a1 AS a, b.a, c.a2, d2 CROSS JOIN e.a CROSS JOIN "F" CROSS JOIN g1.a',
+            'SELECT * FROM a1 AS a /* a */, b.a /* b */, c.a2 /* c.a */, d2 /* d.a */ CROSS JOIN e.a CROSS JOIN "F" /* f-F.A */ CROSS JOIN g1.a /* g */',
+        )
+
+        self.assertEqual(
+            exp.replace_tables(
+                parse_one("select * from example.table", dialect="bigquery"),
+                {"example.table": "`my-project.example.table`"},
+                dialect="bigquery",
+            ).sql(),
+            'SELECT * FROM "my-project".example.table /* example.table */',
         )
 
     def test_expand(self):
@@ -309,9 +320,18 @@ class TestExpressions(unittest.TestCase):
         self.assertEqual(exp.func("bla", 1, "foo").sql(), "BLA(1, foo)")
         self.assertEqual(exp.func("COUNT", exp.Star()).sql(), "COUNT(*)")
         self.assertEqual(exp.func("bloo").sql(), "BLOO()")
+        self.assertEqual(exp.func("concat", exp.convert("a")).sql("duckdb"), "CONCAT('a')")
         self.assertEqual(
             exp.func("locate", "'x'", "'xo'", dialect="hive").sql("hive"), "LOCATE('x', 'xo')"
         )
+        self.assertEqual(
+            exp.func("log", exp.to_identifier("x"), 2, dialect="bigquery").sql("bigquery"),
+            "LOG(x, 2)",
+        )
+        self.assertEqual(
+            exp.func("log", dialect="bigquery", expression="x", this=2).sql("bigquery"),
+            "LOG(x, 2)",
+        )
 
         self.assertIsInstance(exp.func("instr", "x", "b", dialect="mysql"), exp.StrPosition)
         self.assertIsInstance(exp.func("bla", 1, "foo"), exp.Anonymous)
@@ -326,6 +346,15 @@ class TestExpressions(unittest.TestCase):
         with self.assertRaises(ValueError):
             exp.func("abs")
 
+        with self.assertRaises(ValueError) as cm:
+            exp.func("to_hex", dialect="bigquery", this=5)
+
+        self.assertEqual(
+            str(cm.exception),
+            "Unable to convert 'to_hex' into a Func. Either manually construct the Func "
+            "expression of interest or parse the function call.",
+        )
+
     def test_named_selects(self):
         expression = parse_one(
             "SELECT a, b AS B, c + d AS e, *, 'zz', 'zz' AS z FROM foo as bar, baz"
@@ -651,6 +680,10 @@ class TestExpressions(unittest.TestCase):
         self.assertIsInstance(parse_one("*"), exp.Star)
         self.assertEqual(exp.column("a", table="b", db="c", catalog="d"), exp.to_column("d.c.b.a"))
 
+        dot = exp.column("d", "c", "b", "a", fields=["e", "f"])
+        self.assertIsInstance(dot, exp.Dot)
+        self.assertEqual(dot.sql(), "a.b.c.d.e.f")
+
     def test_text(self):
         column = parse_one("a.b.c.d.e")
         self.assertEqual(column.text("expression"), "e")
@@ -791,14 +824,14 @@ FROM foo""",
         )
 
     def test_to_interval(self):
-        self.assertEqual(exp.to_interval("1day").sql(), "INTERVAL '1' day")
-        self.assertEqual(exp.to_interval("  5     months").sql(), "INTERVAL '5' months")
+        self.assertEqual(exp.to_interval("1day").sql(), "INTERVAL '1' DAY")
+        self.assertEqual(exp.to_interval("  5     months").sql(), "INTERVAL '5' MONTHS")
         with self.assertRaises(ValueError):
             exp.to_interval("bla")
 
-        self.assertEqual(exp.to_interval(exp.Literal.string("1day")).sql(), "INTERVAL '1' day")
+        self.assertEqual(exp.to_interval(exp.Literal.string("1day")).sql(), "INTERVAL '1' DAY")
         self.assertEqual(
-            exp.to_interval(exp.Literal.string("  5   months")).sql(), "INTERVAL '5' months"
+            exp.to_interval(exp.Literal.string("  5   months")).sql(), "INTERVAL '5' MONTHS"
         )
         with self.assertRaises(ValueError):
             exp.to_interval(exp.Literal.string("bla"))
@@ -1007,3 +1040,11 @@ FROM foo""",
         query = parse_one("SELECT * FROM foo /* sqlglot.meta x = 1, y = a, z */")
         self.assertEqual(query.find(exp.Table).meta, {"x": "1", "y": "a", "z": True})
         self.assertEqual(query.sql(), "SELECT * FROM foo /* sqlglot.meta x = 1, y = a, z */")
+
+    def test_assert_is(self):
+        parse_one("x").assert_is(exp.Column)
+
+        with self.assertRaisesRegex(
+            AssertionError, "x is not <class 'sqlglot.expressions.Identifier'>\."
+        ):
+            parse_one("x").assert_is(exp.Identifier)
